#!/usr/bin/perl
## 
## Copyright 1999, 2000, 2001, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2011, 2012, 2019 The Regents of the University of California
## All Rights Reserved 
## 
## Permission to use, copy, modify and distribute any part of this
## CoralReef software package for educational, research and non-profit
## purposes, without fee, and without a written agreement is hereby
## granted, provided that the above copyright notice, this paragraph
## and the following paragraphs appear in all copies.
## 
## Those desiring to incorporate this into commercial products or use
## for commercial purposes should contact the Technology Transfer
## Office, University of California, San Diego, 9500 Gilman Drive, La
## Jolla, CA 92093-0910, Ph: (858) 534-5815, FAX: (858) 534-7345.
## 
## IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
## PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
## DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
## SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
## THE POSSIBILITY OF SUCH DAMAGE.
## 
## THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
## UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
## SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
## OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
## OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
## TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
## PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
## ANY PATENT, TRADEMARK OR OTHER RIGHTS.
## 
## The CoralReef software package is developed by the CoralReef
## development team at the University of California, San Diego under
## the Cooperative Association for Internet Data Analysis (CAIDA)
## Program. Support for this effort is provided by the CAIDA grant
## NCR-9711092, DARPA NGI Contract N66001-98-2-8922, DARPA NMS Grant
## N66001-01-1-8909, and by CAIDA members.
## 
## Report bugs and suggestions to coral-bugs@caida.org.
## 


## USAGE:crl_tofr.pl [coral trace file]
##This script converts crl traces to fr format.  Calling it on a file it 
##will dump onto screen.
##
## $Id: crl_tofr.pl,v 1.32 2019/03/28 23:40:29 kkeys Exp $
## $Author: kkeys $
## $Name: release-3-9-5 $
## $Revision: 1.32 $

use File::Basename;
use strict;
my $coral_dir;
BEGIN { $coral_dir = "/usr/local/Coral"; } # This will be edited during build.
use lib "$coral_dir/lib"; #coral global libdir#
use lib "../../lib"; #coral local libdir#
use CRL;

use vars qw($iface $blockhead $cell);


# assign $PNAME to the actual program name
# $script_path is the path to the directory the script is in

my ($script_name, $script_path, $script_suffix) = fileparse($0);
my $PNAME = "$script_name$script_suffix";



## Initialize the data structure unpacking routines.

my $ip_opts = new Unpacker("ip", ["ip_src", "ip_dst", "ip_len",
				  "ihl", "ip_p"]); 

my $tcp_opts = new Unpacker("tcp", ["th_flags", "th_sport", "th_dport"]);

my $udp_opts = new Unpacker("udp", ["uh_sport", "uh_dport"]);



## Command line args check
## change it according to your needs
Coral::quick_start(1);
Coral::read_pkt_init(undef, undef);

my $packet = new Coral::Pkt_result;

my $iface;
while (defined($iface = Coral::read_pkt($packet))) {
    # get timestamp
    my ($sec, $nsec) = Coral::read_clock_sec_nsec($iface, $packet->timestamp);
    my $msec = ($nsec/1000);

    my $ip_record = Coral::get_ip($iface, $packet->packet); 
    if (!defined $ip_record) { next; } # not an ipv4 packet

    # get fr fields
    my ($src, $dst, $ip_len, $ihl, $ip_p) = $ip_opts->unpack($ip_record);
    my ($sport, $dport, $flags) = (0, 0, 0);

    if ($ip_p == 6) {  ##this must be a tcp packet
	($flags, $sport, $dport) = $tcp_opts->unpack($ip_record,$ihl);
    } elsif ($ip_p == 17) { ##this is a udp packet
	($sport, $dport) = $udp_opts->unpack($ip_record,$ihl);
    }

    # output
    my $output = pack("NNNNnCCnn", $sec, $msec, $src, $dst, $ip_len, $ip_p,
		      $flags, $sport, $dport);
    print $output;
}
