#!/usr/bin/perl
## 
## Copyright 1999, 2000, 2001, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2011, 2012, 2019 The Regents of the University of California
## All Rights Reserved 
## 
## Permission to use, copy, modify and distribute any part of this
## CoralReef software package for educational, research and non-profit
## purposes, without fee, and without a written agreement is hereby
## granted, provided that the above copyright notice, this paragraph
## and the following paragraphs appear in all copies.
## 
## Those desiring to incorporate this into commercial products or use
## for commercial purposes should contact the Technology Transfer
## Office, University of California, San Diego, 9500 Gilman Drive, La
## Jolla, CA 92093-0910, Ph: (858) 534-5815, FAX: (858) 534-7345.
## 
## IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
## PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
## DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
## SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
## THE POSSIBILITY OF SUCH DAMAGE.
## 
## THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
## UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
## SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
## OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
## OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
## TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
## PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
## ANY PATENT, TRADEMARK OR OTHER RIGHTS.
## 
## The CoralReef software package is developed by the CoralReef
## development team at the University of California, San Diego under
## the Cooperative Association for Internet Data Analysis (CAIDA)
## Program. Support for this effort is provided by the CAIDA grant
## NCR-9711092, DARPA NGI Contract N66001-98-2-8922, DARPA NMS Grant
## N66001-01-1-8909, and by CAIDA members.
## 
## Report bugs and suggestions to coral-bugs@caida.org.
## 


## $Id: crl_totsh.pl,v 1.28 2019/03/28 23:40:29 kkeys Exp $
## $Author: kkeys $
## $Name: release-3-9-5 $
## $Revision: 1.28 $
##
## USAGE:crl_tofr.pl [coral trace file]
##This script converts crl traces to fr format.  Calling it on a file it 
##will dump onto screen.

use strict;
my $coral_dir;
BEGIN { $coral_dir = "/usr/local/Coral"; } # This will be edited during build.
use lib "$coral_dir/lib"; #coral global libdir#
use lib "../../lib"; #coral local libdir#
use CRL;
use Unpack;

## Initialize the data structure unpacking routines.

Coral::quick_start(1); # do time-sorting
Coral::read_pkt_init(undef, undef);

my $packet = new Coral::Pkt_result;

my $iface;
while (defined($iface = Coral::read_pkt($packet))) {
    my $interface = Coral::interface_get_number($iface);

    # get timestamp
    my ($sec, $nsec) = Coral::read_clock_sec_nsec($iface, $packet->timestamp);
    my $msec = int($nsec/1000);
    $msec = $msec&0xffffff;
    $msec = $msec+ (($interface + 1) <<24);
    my $ip_record = Coral::get_ip($iface, $packet->packet); 

    if (!defined $ip_record) { next; } # not an ipv4 packet

    my $ip_fields = new Unpacker("ip", ["ihl"]);
    my ($ihl) = $ip_fields->unpack($ip_record);

    # get fr fields

    # output
    my $output = pack("NN", $sec, $msec);
    print $output,
	substr($ip_record, 0, 20),
	substr($ip_record, $ihl, 16),
	"\0" x (36 - (length($ip_record) - ($ihl - 20)));
}
