#! /usr/local/bin/perl -w
## 
## Copyright 1999, 2000, 2001, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2011, 2012, 2019 The Regents of the University of California
## All Rights Reserved 
## 
## Permission to use, copy, modify and distribute any part of this
## CoralReef software package for educational, research and non-profit
## purposes, without fee, and without a written agreement is hereby
## granted, provided that the above copyright notice, this paragraph
## and the following paragraphs appear in all copies.
## 
## Those desiring to incorporate this into commercial products or use
## for commercial purposes should contact the Technology Transfer
## Office, University of California, San Diego, 9500 Gilman Drive, La
## Jolla, CA 92093-0910, Ph: (858) 534-5815, FAX: (858) 534-7345.
## 
## IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
## PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
## DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
## SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
## THE POSSIBILITY OF SUCH DAMAGE.
## 
## THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
## UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
## SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
## OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
## OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
## TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
## PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
## ANY PATENT, TRADEMARK OR OTHER RIGHTS.
## 
## The CoralReef software package is developed by the CoralReef
## development team at the University of California, San Diego under
## the Cooperative Association for Internet Data Analysis (CAIDA)
## Program. Support for this effort is provided by the CAIDA grant
## NCR-9711092, DARPA NGI Contract N66001-98-2-8922, DARPA NMS Grant
## N66001-01-1-8909, and by CAIDA members.
## 
## Report bugs and suggestions to coral-bugs@caida.org.
## 
## $Id: crl_ipaddr.pl,v 1.36 2019/03/28 23:40:32 kkeys Exp $
## $Author: kkeys $
## $Name: release-3-9-5 $
## $Revision: 1.36 $

my $coral_dir;
BEGIN { $coral_dir = "/usr/local/Coral"; } # This will be edited during build.
use lib "$coral_dir/lib"; #coral global libdir#
use lib "../../lib"; #coral local libdir#
use CRL;
use strict;

my $ip_opts = new Unpacker("ip", ["ip_src_dotted", "ip_dst_dotted",
				  "ip_p", "ihl", "ip_v"]);

Coral::quick_start();
Coral::read_pkt_init(undef, undef);

my $packet = new Coral::Pkt_result;

my $iface;
while (defined($iface = Coral::read_pkt($packet))) {
    my $ip_packet = new Coral::Pkt_buffer;
    my $ip_record = Coral::get_ip($iface, $packet->packet);
    if (!defined $ip_record) { next; } # not an ipv4 packet

    my $timestamp_ptr = $packet->timestamp;
    my $timestamp = Coral::read_clock_double($iface, $timestamp_ptr);

    my ($ip_src_dotted, $ip_dst_dotted, $ip_p, $ihl) =
	    $ip_opts->unpack($ip_record);

    if (not defined $ip_src_dotted or not defined $ip_dst_dotted or 
	    not defined $ip_p) { next; }

    print "$timestamp\t$ip_src_dotted\t$ip_dst_dotted\t$ip_p";

    if ($ip_p == 4) {
	my ($ipip_src_dotted, $ipip_dst_dotted, $ipip_p, $ipip_ihl, $ipip_v) =
	    $ip_opts->unpack($ip_record, $ihl);
	if (defined $ipip_src_dotted and defined $ipip_dst_dotted
		    and defined $ipip_p and $ipip_v == 4)
	{
	    print "\t$ipip_src_dotted\t$ipip_dst_dotted\t$ipip_p";
	}
    }
    print "\n";
}
