#! /usr/local/bin/perl -w
## 
## Copyright 1999, 2000, 2001, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2011, 2012, 2019 The Regents of the University of California
## All Rights Reserved 
## 
## Permission to use, copy, modify and distribute any part of this
## CoralReef software package for educational, research and non-profit
## purposes, without fee, and without a written agreement is hereby
## granted, provided that the above copyright notice, this paragraph
## and the following paragraphs appear in all copies.
## 
## Those desiring to incorporate this into commercial products or use
## for commercial purposes should contact the Technology Transfer
## Office, University of California, San Diego, 9500 Gilman Drive, La
## Jolla, CA 92093-0910, Ph: (858) 534-5815, FAX: (858) 534-7345.
## 
## IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
## PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
## DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
## SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
## THE POSSIBILITY OF SUCH DAMAGE.
## 
## THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
## UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
## SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
## OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
## OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
## TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
## PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
## ANY PATENT, TRADEMARK OR OTHER RIGHTS.
## 
## The CoralReef software package is developed by the CoralReef
## development team at the University of California, San Diego under
## the Cooperative Association for Internet Data Analysis (CAIDA)
## Program. Support for this effort is provided by the CAIDA grant
## NCR-9711092, DARPA NGI Contract N66001-98-2-8922, DARPA NMS Grant
## N66001-01-1-8909, and by CAIDA members.
## 
## Report bugs and suggestions to coral-bugs@caida.org.
## 


## $Id: crl_llcsnap.pl,v 1.38 2019/03/28 23:40:32 kkeys Exp $
## $Author: kkeys $
## $Name: release-3-9-5 $
## $Revision: 1.38 $
##
## This script will print out the all of the combinations of atm headers
## and llc/snap values in the trace, separated out by interface.  The
## number of occurances is shown first.
## Based originally on oc32asc.pl by Hans-Werner Braun
##
## USAGE:  crl_llcsnap.pl <Coral Dump>
##

my $coral_dir;
BEGIN { $coral_dir = "/usr/local/Coral"; } # This will be edited during build.
use lib "$coral_dir/lib"; #coral global libdir#
use lib "../../lib"; #coral local libdir#
use CRL;
use strict;

my $atm_opts = new Unpacker("atm", ["vp", "vc"]);
Coral::set_iomode(0, 0, 48, 1);
Coral::set_duration(0);
Coral::quick_start(0, 0, $Coral::API_CELL);

my %atm_llc_snap;
my $iface;
my $cell;

while (defined($iface = Coral::read_cell_all(undef, \$cell))
	&& defined($cell))
{
    my $interface = Coral::interface_get_number($iface);
    my $header = $cell->header($iface);
    my ($vp, $vc) = $atm_opts->unpack($header);
    my $payload = $cell->payload($iface);
    my $llcsnap = substr($cell->payload($iface), 0, 8);
    my $cell_str = $llcsnap . "$vp:$vc";
    $atm_llc_snap{$interface}{$cell_str}++;
}

my $llcsnap_opts = new Unpacker("llcsnap", ["llc_dsap", "llc_ssap", "llc_cntl",
					    "snap_org", "snap_type"]);

foreach my $interface (sort { $a <=> $b } keys %atm_llc_snap) {
    print "# Interface: $interface\n";
    print "Count\tFields:\n";

    foreach my $header (sort { $atm_llc_snap{$interface}{$b} <=>
	    $atm_llc_snap{$interface}{$a} } keys %{$atm_llc_snap{$interface}})
    {
	my $cnt = $atm_llc_snap{$interface}{$header};
	my $llcsnap = substr($header, 0, 8);
	my ($vp, $vc) = split ':', substr($header, 8);
	my ($llc_dsap, $llc_ssap, $llc_cntl, $snap_org, $snap_type) =
	    $llcsnap_opts->unpack($llcsnap);

	printf("%.f\tvpvc=%d:%d dsap=0x%02x ssap=0x%02x cntl=0x%02x org=0x%06x"
		. " type=0x%04x\n",
	       $cnt, $vp, $vc,
	       $llc_dsap, $llc_ssap, $llc_cntl, $snap_org, $snap_type);
    }
}
