#! /usr/local/bin/perl5
## 
## Copyright 1999, 2000, 2001, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2011, 2012, 2019 The Regents of the University of California
## All Rights Reserved 
## 
## Permission to use, copy, modify and distribute any part of this
## CoralReef software package for educational, research and non-profit
## purposes, without fee, and without a written agreement is hereby
## granted, provided that the above copyright notice, this paragraph
## and the following paragraphs appear in all copies.
## 
## Those desiring to incorporate this into commercial products or use
## for commercial purposes should contact the Technology Transfer
## Office, University of California, San Diego, 9500 Gilman Drive, La
## Jolla, CA 92093-0910, Ph: (858) 534-5815, FAX: (858) 534-7345.
## 
## IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
## PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
## DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
## SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
## THE POSSIBILITY OF SUCH DAMAGE.
## 
## THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
## UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
## SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
## OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
## OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
## TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
## PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
## ANY PATENT, TRADEMARK OR OTHER RIGHTS.
## 
## The CoralReef software package is developed by the CoralReef
## development team at the University of California, San Diego under
## the Cooperative Association for Internet Data Analysis (CAIDA)
## Program.  Support for this effort is provided by the CAIDA grant
## NCR-9711092, DARPA NGI Contract N66001-98-2-8922, DARPA NMS Grant
## N66001-01-1-8909, and by CAIDA members.
## 
## Report bugs and suggestions to coral-bugs@caida.org.
## 

## $Id: merge_protos.pl,v 1.16 2019/03/28 23:40:38 kkeys Exp $ $Name: release-3-9-5 $

use IO::File;

my @proto_files = @ARGV;
my @file_handles;

foreach my $proto_file (@proto_files) {
    my $handle = new IO::File $proto_file;
    die "Couldn't open $proto_file for reading: $!" if not $handle;
    push @file_handles, $handle;
}

die "No files opened" if @file_handles == 0;

while (1) {
    my ($prev_rrd_line, $out_str);
    my ($val_tot, $byte_tot, $pkt_tot, $flow_tot);
    foreach my $handle (@file_handles) {
	my $line = <$handle>;
	goto DONE if not defined $line;
	if ($line =~ m#value> (\d\S*) <#) {
	    $val_tot += $1;
	    my $val_str = sprintf "%.10e", $val_tot;
	    $line =~ s#value> (\d\S*) <#value> $val_str <#;
	} elsif ($line =~ m#<unknown_datapoints> (\d\S*) <#) {
	    $line =~ s#unknown_datapoints> (\d\S*) <#unknown_datapoints> 0 <#;
	} elsif ($line =~ m#<v> (\d\S*) </v><v> (\d\S*) </v><v> (\d\S*) </v>#) {
	    $byte_tot += $1;
	    $pkt_tot += $2;
	    $flow_tot += $3;
	} else {
	    if ($line =~ m#<last_ds>  </last_ds>#) {
		$line =~ s#<last_ds>  </last_ds>#<last_ds> UNKN </last_ds>#;
	    }
	    if ($line =~ m#<v> NaN </v><v> NaN </v><v> NaN </v># or
		$line =~ m#value> NaN <#)
	    {
		# Ignore
	    } elsif ($prev_rrd_line and $line ne $prev_rrd_line) {
		die "Conflicting lines between RRD files:\n" .
		    "$prev_rrd_line\nand\n\n$line\n" .
		    "Cannot continue";
	    }
	}
	$prev_rrd_line = $line;
    }
    if ($byte_tot and $pkt_tot and $flow_tot) {
	my $byte_str = sprintf "%.10e", $byte_tot;
	my $pkt_str = sprintf "%.10e", $pkt_tot;
	my $flow_str = sprintf "%.10e", $flow_tot;
	$prev_rrd_line =~ s#<v> .* </v><v> .* </v><v> .* </v>#<v> $byte_str </v><v> $pkt_str </v><v> $flow_str </v>#;
    }
    print $prev_rrd_line;
}
DONE:
