/* 
 * Copyright 1999, 2000, 2001, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2011, 2012, 2019 The Regents of the University of California
 * All Rights Reserved 
 * 
 * Permission to use, copy, modify and distribute any part of this
 * CoralReef software package for educational, research and non-profit
 * purposes, without fee, and without a written agreement is hereby
 * granted, provided that the above copyright notice, this paragraph
 * and the following paragraphs appear in all copies.
 * 
 * Those desiring to incorporate this into commercial products or use
 * for commercial purposes should contact the Technology Transfer
 * Office, University of California, San Diego, 9500 Gilman Drive, La
 * Jolla, CA 92093-0910, Ph: (858) 534-5815, FAX: (858) 534-7345.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
 * PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
 * DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
 * UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
 * OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
 * OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
 * ANY PATENT, TRADEMARK OR OTHER RIGHTS.
 * 
 * The CoralReef software package is developed by the CoralReef
 * development team at the University of California, San Diego under
 * the Cooperative Association for Internet Data Analysis (CAIDA)
 * Program. Support for this effort is provided by the CAIDA grant
 * NCR-9711092, DARPA NGI Contract N66001-98-2-8922, DARPA NMS Grant
 * N66001-01-1-8909, and by CAIDA members.
 * 
 * Report bugs and suggestions to coral-bugs@caida.org.
 */

/* $Id: crl_print.c,v 1.31 2019/03/28 23:40:40 kkeys Exp $ */


static const char RCSid[]="$Id: crl_print.c,v 1.31 2019/03/28 23:40:40 kkeys Exp $";

#include "config.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "libcoral.h"

static void usage(const char *name)
{
    coral_usage(name, "[-r] <source>...\n"
	"-r    print raw (unformatted and uncorrected) cells\n");
}

int main (int argc, char *argv[])
{
    coral_iface_t *iface;
    coral_source_t *src, *nextsrc;
    coral_atm_cell_t *cell;	/* pointer to cell */
    u_int count = 0;
    int opt, raw = 0;

    extern char *optarg;
    extern int optind;

    coral_set_api(CORAL_API_CELL);
    coral_set_iomode(0, CORAL_RX_UNKNOWN, -1, 0);
    coral_set_duration(0);

    while ((opt = getopt(argc, argv, "C:r")) != -1) {
        switch (opt) {
        case 'C':
            if (coral_config_command(optarg) < 0) {
		usage(argv[0]);
                exit(-1);
	    }
            break;
        case 'r':
	    coral_set_options(0, CORAL_OPT_RAW_TIME);
            raw = 1;
            break;
        default:
	    usage(argv[0]);
            exit(-1);
        }
    }

    while (optind < argc) {
        if (!coral_new_source(argv[optind]))
            exit(-1);
        optind++;
    }

    if (coral_open_all() < 0)
        exit(-1);

    if (coral_start_all() < 0)
        exit(0);

    nextsrc = NULL;
    for (src = coral_next_source(NULL); src; src = nextsrc) {
	nextsrc = coral_next_source(src);
	iface = coral_next_src_iface(src, NULL);
	if (coral_interface_get_physical(iface) != CORAL_PHY_ATM) {
	    coral_diag(0, ("%s: %s is not an ATM source, ignoring it.\n",
		argv[0], coral_source_get_filename(src)));
	    coral_close(src);
	}
    }

    while ((iface = coral_read_cell_all(NULL, &cell, NULL))) {
	printf("cell %d, interface %d\n",
	    count, coral_interface_get_number(iface));
	if (raw)
	    coral_print_data(8, (u_char*)cell, coral_cell_size(iface));
	else
	    coral_print_cell(8, iface, cell);
	count++;
    }

    if (errno) exit(errno);
    coral_stop_all();
    coral_close_all();
    exit(0);
}
