#!/usr/local/bin/perl -w
## Copyright 2000, 2001, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2011, 2012, 2019 The Regents of the University of California
## All Rights Reserved 
## 
## Permission to use, copy, modify and distribute any part of this
## CoralReef software package for educational, research and non-profit
## purposes, without fee, and without a written agreement is hereby
## granted, provided that the above copyright notice, this paragraph
## and the following paragraphs appear in all copies.
## 
## Those desiring to incorporate this into commercial products or use
## for commercial purposes should contact the Technology Transfer
## Office, University of California, San Diego, 9500 Gilman Drive, La
## Jolla, CA 92093-0910, Ph: (858) 534-5815, FAX: (858) 534-7345.
## 
## IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
## PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
## DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
## SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
## THE POSSIBILITY OF SUCH DAMAGE.
## 
## THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
## UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
## SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
## OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
## OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
## TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
## PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
## ANY PATENT, TRADEMARK OR OTHER RIGHTS.
## 
## The CoralReef software package is developed by the CoralReef
## development team at the University of California, San Diego under
## the Cooperative Association for Internet Data Analysis (CAIDA)
## Program. Support for this effort is provided by the CAIDA grant
## NCR-9711092, DARPA NGI Contract N66001-98-2-8922, DARPA NMS Grant
## N66001-01-1-8909, and by CAIDA members.
## 
## Report bugs and suggestions to coral-bugs@caida.org.
## 

use Socket qw(inet_aton inet_ntoa);
use vars qw($last_net $last_len);

while (<>) {
    chomp;
    s/#.*//g;
    if (/([\d|\.]+)\/(\d+)\t0x([^\s]+)\t(\d+)\t(.*)/) { 
	my ($network, $length, $network2, $length2, $as) 
	    = ($1, $2, $3, $4, $5);

	my $hex_net = unpack "N", inet_aton($network);
	my $hex_net2 = hex $network2;
	my $masked_net = $hex_net & (0xFFFFFFFF<<(32-$length));
	if ($hex_net != $hex_net2) {
	    print STDERR "$ARGV (line $.): networks don't match\n";
	}
	if ($length != $length2) {
	    print STDERR "$ARGV (line $.): mask lengths don't match\n";
	}
	if ($hex_net != $masked_net) {
	    print STDERR "$ARGV (line $.): network doesn't match mask length\n";
	}
	if (defined $last_net and defined $last_len and 
	    ($hex_net < $last_net or
	    $hex_net == $last_net and $length > $last_len))
	{
	    print STDERR "$ARGV (line $.): networks aren't sorted correctly\n";
	}
	$last_net = $hex_net;
	$last_len = $length;
    }
    else {
#	if (/[^\s]/) {
	print STDERR "$ARGV (line $.) could't parse '$_'\n";
    }
} continue {
    close ARGV if eof;
}
