/* 
 * Copyright 1999, 2000, 2001, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2011, 2012, 2019 The Regents of the University of California
 * All Rights Reserved 
 * 
 * Permission to use, copy, modify and distribute any part of this
 * CoralReef software package for educational, research and non-profit
 * purposes, without fee, and without a written agreement is hereby
 * granted, provided that the above copyright notice, this paragraph
 * and the following paragraphs appear in all copies.
 * 
 * Those desiring to incorporate this into commercial products or use
 * for commercial purposes should contact the Technology Transfer
 * Office, University of California, San Diego, 9500 Gilman Drive, La
 * Jolla, CA 92093-0910, Ph: (858) 534-5815, FAX: (858) 534-7345.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
 * PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
 * DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
 * UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
 * OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
 * OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
 * ANY PATENT, TRADEMARK OR OTHER RIGHTS.
 * 
 * The CoralReef software package is developed by the CoralReef
 * development team at the University of California, San Diego under
 * the Cooperative Association for Internet Data Analysis (CAIDA)
 * Program. Support for this effort is provided by the CAIDA grant
 * NCR-9711092, DARPA NGI Contract N66001-98-2-8922, DARPA NMS Grant
 * N66001-01-1-8909, and by CAIDA members.
 * 
 * Report bugs and suggestions to coral-bugs@caida.org.
 */
/*
 * $Name: release-3-9-5 $
 * $Id: pat_func.h,v 1.24 2019/03/28 23:40:47 kkeys Exp $
 */
#ifndef PAT_FUNC_H
#define PAT_FUNC_H

#include <string>
#include "Ipv4Network.hh"
#ifdef AF_INET6
# include "Ipv6Network.hh"
#endif
#include "IpPrefixPatricia.hh"

typedef Ipv4PrefixPatricia<std::string> ASFinder;

ASFinder * new_as_finder();
int load_file_text(ASFinder * finder, const char * digested_bgp_filename);
const char * get_as(ASFinder * finder, const char * ip_str, char * net,
			int * masklen);
const char * get_as_raw(ASFinder * finder, struct in_addr * ip, char * net,
			int * masklen);
void delete_as_finder(ASFinder * finder);

#ifdef AF_INET6
typedef Ipv6PrefixPatricia<std::string> ASFinder6;
#else /* no AF_INET6 */
/* There's no way to do conditional compilation in swig-generated ASFinder.cc,
 * so we must define (a version of) ASFinder6 and declare the functions so
 * ASFinder.cc will compile.  Any perl script that tries to use them will
 * give a dynamic linker error.
 */
typedef struct ASFinder6 ASFinder6;
#endif /* AF_INET6 */

ASFinder6 * new_as_finder6();
int load_file_text6(ASFinder6 * finder, const char * digested_bgp_filename);
const char * get_as6(ASFinder6 * finder, const char * ip_str, char * net,
			int * masklen);
const char * get_as_raw6(ASFinder6 * finder, struct in6_addr * ip, char * net,
			int * masklen);
void delete_as_finder6(ASFinder6 * finder);

#endif
