 /* 
  * Copyright 2001, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2011, 2012, 2019 The Regents of the University of California
  * All Rights Reserved 
  * 
  * Permission to use, copy, modify and distribute any part of this
  * CoralReef software package for educational, research and non-profit
  * purposes, without fee, and without a written agreement is hereby
  * granted, provided that the above copyright notice, this paragraph
  * and the following paragraphs appear in all copies.
  * 
  * Those desiring to incorporate this into commercial products or use
  * for commercial purposes should contact the Technology Transfer
  * Office, University of California, San Diego, 9500 Gilman Drive, La
  * Jolla, CA 92093-0910, Ph: (858) 534-5815, FAX: (858) 534-7345.
  * 
  * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
  * PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
  * DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
  * THE POSSIBILITY OF SUCH DAMAGE.
  * 
  * THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
  * UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
  * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
  * OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
  * OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
  * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
  * PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
  * ANY PATENT, TRADEMARK OR OTHER RIGHTS.
  * 
  * The CoralReef software package is developed by the CoralReef
  * development team at the University of California, San Diego under
  * the Cooperative Association for Internet Data Analysis (CAIDA)
  * Program. Support for this effort is provided by the CAIDA grant
  * NCR-9711092, DARPA NGI Contract N66001-98-2-8922, DARPA NMS Grant
  * N66001-01-1-8909, and by CAIDA members.
  * 
  * Report bugs and suggestions to coral-bugs@caida.org.
  */

%{
//===========================================================================
// @(#) $Name: release-3-9-5 $
// @(#) $Id: AppPorts.lex,v 1.22 2019/03/28 23:40:47 kkeys Exp $
//===========================================================================
extern "C" {
#include <errno.h>
}

#include "AppPorts.h"
#include <fstream>
#include <cstdio>
#include <string>
#include <cstring>

using namespace std;

static const string rcsid = "@(#) $Name: release-3-9-5 $ $Id: AppPorts.lex,v 1.22 2019/03/28 23:40:47 kkeys Exp $";
static bool is_src = false;

%}

Octet		[0-9]{1,3}
IpAddress	{Octet}\.{Octet}\.{Octet}\.{Octet}
FileName	[^#\*\&\^\$ \t\n\!\~\`\'\"\;]*
String		([ \t]*[^# \t\n]+)*
Wildnum		(\*|[0-9]+)
OptComment	[ \t]*(#.*)?\n

%s  DESC
%s  NAME
%s  GROUP
%s  NET
%s  SLASH
%s  LENGTH
%s  NOLENGTH
%s  PORT
%s  NOPORT
%s  MOREPORT
%s  SECPORT
%s  SYM
%s  PORTSOK
%s  PROTO
%s  CONTRIB
%s  DATE
%s  NOTES
%s  REF
%s  URL
%s  PRIORITY
%s  EOL
%s  ENDRULE
%x  ERROR

%option noyywrap
%option yylineno

%%

<INITIAL>^[ \t]*description:[ \t]*	{BEGIN(DESC);}
<INITIAL>^[ \t]*name:[ \t]*		{BEGIN(NAME);}
<INITIAL>^[ \t]*group:[ \t]*		{BEGIN(GROUP);}
<INITIAL>^[ \t]*srcnet:[ \t]*		{is_src = true; BEGIN(NET);}
<INITIAL>^[ \t]*dstnet:[ \t]*		{is_src = false; BEGIN(NET);}
<INITIAL>^[ \t]*sport:[ \t]*		{is_src = true; BEGIN(PORT);}
<INITIAL>^[ \t]*dport:[ \t]*		{is_src = false; BEGIN(PORT);}
<INITIAL>^[ \t]*sym:[ \t]*		{BEGIN(SYM);}
<INITIAL>^[ \t]*ports_ok:[ \t]*		{BEGIN(PORTSOK);}
<INITIAL>^[ \t]*protocol:[ \t]*		{BEGIN(PROTO);}
<INITIAL>^[ \t]*priority:[ \t]*		{BEGIN(PRIORITY);}
<INITIAL>^[ \t]*contributor:[ \t]*	{BEGIN(CONTRIB);}
<INITIAL>^[ \t]*date:[ \t]*		{BEGIN(DATE);}
<INITIAL>^[ \t]*notes:[ \t]*		{BEGIN(NOTES);}
<INITIAL>^[ \t]*reference:[ \t]*	{BEGIN(REF);}
<INITIAL>^[ \t]*url:[ \t]*		{BEGIN(URL);}
<INITIAL>^#[^\n]*\n			/* skip comment */
<INITIAL>^[ \t]*\n			{ return ENDRULE; }

<DESC>{String}			{ BEGIN(EOL); return DESC; }
<NAME>{FileName}		{ BEGIN(EOL); return NAME; }
<GROUP>{FileName}		{ BEGIN(EOL); return GROUP; }

<NET>{IpAddress}		{ BEGIN(SLASH); return NET; }
<SLASH>"/"			{ BEGIN(LENGTH); }
<SLASH>[ \t]*,[ \t]*		{ BEGIN(NET); return NOLENGTH; }
<SLASH>{OptComment}		{ BEGIN(INITIAL); return NOLENGTH; }
<LENGTH>[0-9]+[ \t]*,[ \t]*	{ BEGIN(NET); return LENGTH; }
<LENGTH>[0-9]+			{ BEGIN(EOL); return LENGTH; }

<PORT>{Wildnum}[ \t]*		{ BEGIN(MOREPORT); return PORT; }
<PORT>none			{ BEGIN(EOL); return NOPORT; }
<MOREPORT>-[ \t]*		{ BEGIN(SECPORT); }
<MOREPORT>,[ \t]*		{ BEGIN(PORT); }
<MOREPORT>{OptComment}		{ BEGIN(INITIAL); }
<SECPORT>[0-9]+[ \t]*		{ BEGIN(MOREPORT); return SECPORT; }

<SYM>[01]			{ BEGIN(EOL); return SYM; }
<PORTSOK>[01]			{ BEGIN(EOL); }
<PROTO>{Wildnum},[ \t]*		{ return PROTO; }
<PROTO>{Wildnum}		{ BEGIN(EOL); return PROTO; }
<PRIORITY>[0-9]+		{ BEGIN(EOL); return PRIORITY; }

<CONTRIB>{String}		{ BEGIN(EOL); return CONTRIB; }
<DATE>{String}			{ BEGIN(EOL); return DATE; }
<NOTES>{String}			{ BEGIN(EOL); return NOTES; }
<REF>{String}			{ BEGIN(EOL); return REF; }
<URL>{String}			{ BEGIN(EOL); return URL; }

<*>[ \t]+			{ }
\n				{ BEGIN(INITIAL); }
<EOL>{OptComment}		{ BEGIN(INITIAL); }

.				{ return ERROR; }

%%

bool AppPorts::load_rules(const string& fileName)
{
    int rc, int_val;
    bool ok = false;
    const int max_prefix_len = 32;
    Ipv4Network network;
    bool ready = false;

    yyin = fopen(fileName.c_str(), "r");
    if (!yyin) {
	cerr << fileName << ": " << strerror(errno) << endl;
	return false;
    }
    rule_type *rule = new rule_type;
    unsigned int port;
    int16_t proto;

    while ((rc = yylex()) != 0) {
	switch (rc) {
	case DESC:
	    rule->data.desc = yytext;
	    break;
	case NAME:
	    rule->data.name = yytext;
	    break;
	case GROUP:
	    rule->data.group = yytext;
	    break;
	case NET:
	    try {
		network.net.parse(yytext);
	    } catch (std::runtime_error e) {
		cerr << fileName << ", line " << yylineno <<
		    ": invalid address" << endl;
		ok = false;
		break;
	    }
	    ok = true;
	    network.maskLen = 32;
	    break;
	case LENGTH:
	    if (!ok) break;
	    network.maskLen = strtoul(yytext,NULL,10);
	    // fall through
	case NOLENGTH:
	    if (network.maskLen > max_prefix_len) {
		cerr << fileName << ", line " << yylineno <<
		    ": invalid prefix length " << int(network.maskLen) << endl;
		break;
	    }
	    if (is_src) {
		rule->src_nets.push_back(network);
	    } else {
		rule->dst_nets.push_back(network);
	    }
	    break;
	case PORT:
	    rule->check_ports = true;
	    if (yytext[0] == '*') {
		if (is_src) {
		    rule->src_ports.push_back(all_ports());
		} else {
		    rule->dst_ports.push_back(all_ports());
		}
	    } else {
		port = strtoul(yytext, NULL, 10);
		if (is_src) {
		    rule->src_ports.push_back(num_range(port,port));
		} else {
		    rule->dst_ports.push_back(num_range(port,port));
		}
	    }
	    break;
	case NOPORT:
	    rule->check_ports = true;
	    rule->no_ports = true;
	    break;
	case SECPORT: // XXX If this is called in the wrong place, we're screwed
	    port = strtoul(yytext, NULL, 10);
	    if (is_src) {
		rule->src_ports.back().end = port;
	    } else {
		rule->dst_ports.back().end = port;
	    }
	    break;
	case SYM:
	    rule->symmetric = atoi(yytext);
	    break;
	case PROTO:
	    if (yytext[0] == '*') {
		proto = -1;
	    } else {
		proto = atoi(yytext);
	    }
	    rule->protocols.push_back(proto);
	    break;
	case PRIORITY:
	    rule->priority = atoi(yytext);
	    break;
	case CONTRIB:
	    rule->data.contrib = yytext;
	    break;
	case DATE:
	    rule->data.date = yytext;
	    break;
	case NOTES:
	    rule->data.notes = yytext;
	    break;
	case REF:
	    rule->data.reference = yytext;
	    break;
	case URL:
	    rule->data.url = yytext;
	    break;
	case ENDRULE:
	    if (!rule->data.name.empty()) {
		add_rule(*rule);
	    }
	    delete rule;
	    rule = new rule_type;
	    break;
	default:
	    cerr << fileName << ", line " << yylineno << ": invalid value\n";
	    delete rule;
	    fclose(yyin);
	    return false;
	}
    }
    if (!rule->data.name.empty()) {
	add_rule(*rule);
    }

    delete rule;
    fclose(yyin);
    return true;
}

bool AppPorts::dump_rules(const string& filename)
{
    ostream * os;
    ofstream outfile;
    if (!filename.compare("-")) {
	os = &cout;
    } else {
	outfile.open(filename.c_str());
	if (!outfile) {
	    cerr << filename << ": " << strerror(errno) << endl;
	    return false;
	}
	os = &outfile;
    }
    iterator curr;
    int size;
    for (curr = rules.begin(); curr != rules.end(); curr++) {
	*os << "description:\t" << curr->data.desc << "\n";
	*os << "name:\t" << curr->data.name << "\n";
	*os << "group:\t" << curr->data.group << "\n";
	if ((size = curr->src_nets.size())) {
	    *os << "srcnet:\t";
	    for (int i = 0; i < size; i++) {
		if (i != 0) { *os << ", "; }
		*os << curr->src_nets[i];
	    }
	    *os << "\n";
	}
	if ((size = curr->dst_nets.size())) {
	    *os << "dstnet:\t";
	    for (int i = 0; i < size; i++) {
		if (i != 0) { *os << ", "; }
		*os << curr->dst_nets[i];
	    }
	    *os << "\n";
	}
	if (curr->check_ports) {
	    *os << "sport:\t";
	    if (curr->no_ports) {
		*os << "none";
	    } else {
		for (int i = 0; i < curr->src_ports.size(); i++) {
		    if (i != 0) { *os << ", "; }
		    *os << ports_to_str(curr->src_ports[i]);
		}
	    }
	    *os << "\n";
	    *os << "dport:\t";
	    if (curr->no_ports) {
		*os << "none";
	    } else {
		for (int i = 0; i < curr->dst_ports.size(); i++) {
		    if (i != 0) { *os << ", "; }
		    *os << ports_to_str(curr->dst_ports[i]);
		}
	    }
	    *os << "\n";
	}
	*os << "sym:\t" << curr->symmetric << "\n";
	*os << "protocol:\t";
	for (int i = 0; i < curr->protocols.size(); i++) {
	    if (i != 0) { *os << ", "; }
	    *os << proto_to_str(curr->protocols[i]);
	}
	*os << "\n";
	*os << "priority:\t" << curr->priority << "\n";
	if (!curr->data.contrib.empty()) {
	    *os << "contributor:\t" << curr->data.contrib << "\n";
	}
	if (!curr->data.date.empty()) {
	    *os << "date:\t" << curr->data.date << "\n";
	}
	if (!curr->data.notes.empty()) {
	    *os << "notes:\t" << curr->data.notes << "\n";
	}
	if (!curr->data.reference.empty()) {
	    *os << "reference:\t" << curr->data.reference << "\n";
	}
	if (!curr->data.url.empty()) {
	    *os << "url:\t" << curr->data.url << "\n";
	}
	*os << "\n";
    }
}
