package CAIDA::CRL_report;
require Exporter;

## CRL_report.pm is a package for generating and manipulating report files
##	generated by crl_hist.pl and crl_bycountry.pl
##  
## $Id: CRL_report.pm,v 2.11 2004/11/13 00:25:48 kkeys Exp $ $Name: release-3-9-5 $
##
## This software product is developed by Sean McCreary and David Moore,
## and copyrighted(C) 1998 by the University of California, San Diego
## (UCSD), with all rights reserved. UCSD administers the CAIDA grant,
## NCR-9711092, under which part of this code was developed.
##
## There is no charge for this software. You can redistribute it and/or
## modify it under the terms of the GNU General Public License, v. 2 dated
## June 1991 which is incorporated by reference herein. This software is
## distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF MERCHANTABILITY
## OR FITNESS FOR A PARTICULAR PURPOSE or that the use of it will not
## infringe on any third party's intellectual property rights.
##
## You should have received a copy of the GNU GPL along with this program.
##
## Contact: coral-info@caida.org
##

$VERSION = 1.0;

@ISA = qw(Exporter);
@EXPORT_OK = qw(
                &sum_rows
		&get_counts
		&inc_counts
		&get_keys
		&parse_header
		&parse_plen_hist
		&parse_proto_table
		&parse_port_table
		&parse_matrix
		&parse_4_table
		&parse_5_hist
		&print_header
		&print_plen_hist
		&print_proto_table
		&print_port_table
		&print_matrix
                &print_4_table
                &print_4_table_top10
                &print_5_hist
		&print_html_header
		&print_html_proto_table
		&print_html_tcp_port_table
		&print_html_udp_port_table
		&print_html_matrix
		);

use strict;
use Carp;
use vars qw($package $debug $cvs_Id $cvs_Author $cvs_Name $cvs_Revision);

$cvs_Id = '$Id: CRL_report.pm,v 2.11 2004/11/13 00:25:48 kkeys Exp $';
$cvs_Author = '$Author: kkeys $';
$cvs_Name = '$Name: release-3-9-5 $';
$cvs_Revision = '$Revision: 2.11 $';

$debug = 0;			# turn debug mesg off by default

## Globals, leftover from old header format code
my (@traces);
my ($packets,$bytes);
my ($total_packets,$total_bytes);
my $total_time = 0.0;
my ($opcount,$fragcount,$tcpfragcount,$udpfragcount,$nofragcount);
my ($nonip);
my ($counts_byproto,$matrix_tcp_byport,$matrix_udp_byport);

##===========================================================================
##
## Subroutines

##
## Sums rows from supplied table across list of row keys
##
sub sum_rows($@) {
    my ($table, @keys) = @_;
    my @datasum;

    foreach my $key ( @keys ) {
        my @data = get_counts( $table, $key );
        for ( my $i = $[;  $i <= $#data; $i++ ) {
            if ( ! defined $datasum[$i] ) {
                $datasum[$i] = 0;
            }
            if ( defined $data[$i] ) {
                $datasum[$i] += $data[$i];
            }
        }
    }

    return @datasum;
}

sub get_counts_hash ($$) {
    my ($table_ref, $key) = @_;

    if ( defined ${$table_ref}{$key} ) {
        warn "Value for :$key: is @{${$table_ref}{$key}}\n" if $debug > 9;
        return @{${$table_ref}{$key}};
    } else {
        warn "No value for :$key:\n" if $debug > 9;
        return ();
    }
}

sub inc_counts_hash ($$$) {
    my ($table_ref, $key, $data) = @_;

    my @datasum = get_counts($table_ref, $key);
    for(my $i=0; $i < @{$data}; $i++) {
        if (defined $datasum[$i]) {
            ${$data}[$i] += $datasum[$i];
        }
    }
    ${$table_ref}{$key} = $data;
    warn "New value for :$key: is @{${$table_ref}{$key}}\n" if $debug > 9;

    return $table_ref;
}

sub get_keys_hash ($) {
    my ($table_ref) = @_;
    return keys %{$table_ref};
}

sub get_counts_list ($$) {
    my ($table_ref, $key) = @_;

    if ( defined ${$table_ref}[$key] ) {
        return @{${$table_ref}[$key]};
    } else {
        return ();
    }
}

sub inc_counts_list ($$$) {
    my ($table_ref, $key, $data) = @_;

    my @datasum = get_counts_list($table_ref, $key);
    for(my $i=0; $i < @{$data}; $i++) {
        if (defined $datasum[$i]) {
            ${$data}[$i] += $datasum[$i];
        }
    }
    if ( $key >= 0 ) {
        ${$table_ref}[$key] = $data;
    } else {
        carp "Negative key value $key!  Ignoring request";
    }
    return $table_ref;
}

sub get_keys_list ($) {
    my ($table_ref) = @_;
    
    return (0..$#{$table_ref});
}

sub get_counts ($$) {
    my ($table_ref, $key) = @_;
    my ($reftype);

    $reftype = ref($table_ref);
    if ( $reftype eq "ARRAY" ) {
        return get_counts_list($table_ref, $key);
    } elsif ( $reftype eq "HASH" ) {
        return get_counts_hash($table_ref, $key);
    } elsif ( $reftype eq "" ) { # Not a reference, so create a new hash
        return get_counts_hash($table_ref, $key);
    } else {
        carp "Don't know how to handle reference type $reftype";
    }
}

sub inc_counts ($$$) {
    my ($table_ref, $key, $data) = @_;
    my ($reftype);

    $reftype = ref($table_ref);
    if ( $reftype eq "ARRAY" ) {
        return inc_counts_list($table_ref, $key, $data);
    } elsif ( $reftype eq "HASH" ) {
        return inc_counts_hash($table_ref, $key, $data);
    } elsif ( $reftype eq "" ) { # Not a reference, so create a new hash
        return inc_counts_hash($table_ref, $key, $data);
    } else {
        carp "Don't know how to handle reference type $reftype";
    }
}

sub get_keys ($) {
    my ($table_ref) = @_;
    my ($reftype);

    $reftype = ref($table_ref);
    if ( $reftype eq "ARRAY" ) {
        return get_keys_list($table_ref);
    } elsif ( $reftype eq "HASH" ) {
        return get_keys_hash($table_ref);
    } elsif ( $reftype eq "" ) { # Not a reference, so create a new hash
        return get_keys_hash($table_ref);
    } else {
        carp "Don't know how to handle reference type $reftype";
    }
}

my $localref;
sub by_bytes () {
    my ($packets,$bytes_a,$bytes_b);
    ($packets, $bytes_a) = get_counts($localref, $a);
    ($packets, $bytes_b) = get_counts($localref, $b);
    return $bytes_b <=> $bytes_a;
}

#
# my_cmp puts string values last
#
sub my_cmp ($$) {
    my ($a, $b) = @_;

## Turn off warnings
$^W = 0;
#    if ( $a !~ /^\d+$/ ) {
    if ( $a == 0 ) {
#        if ( $b !~ /^\d+$/ ) {
        if ( $b == 0 ) {
            return $a cmp $b;
        } else {
            return 1;
        }
    } else {
#        if ( $b !~ /^\d+$/ ) {
        if ( $b == 0 ) {
            return -1;
        } else {
            return $a <=> $b;
        }
    }  
## Turn warnings back on
$^W = 1;
}

sub by_pairs {
    my ($src_a, $dst_a) = split /\t/, $a;
    my ($src_b, $dst_b) = split /\t/, $b;
    my ($src_cmp) = my_cmp($src_a, $src_b);
    
    if ($src_cmp != 0) {
        return $src_cmp;
    } else {
        return my_cmp($dst_a, $dst_b);
    }
}

##===========================================================================

#
#
#
#

sub parse_header ($$;$) {
    my ($infile, $header, $regex) = @_;
    my ($line, $rev);

## Look for major version number of program that generated report
    while (<$infile>) {
## Add lines up to the first CVS revision tag to the unstructured header
        ${$header}{"unstructured"} .= $_;
        if (/^#(?:\$)Revision: (\d+\.\d+)(?:\.\d+)* \$/) {
            $rev = $1;
            last;
        }
    }
    if ( !defined($rev) ) { 
        croak "Couldn't find revision of report file!";
    }

    if ($rev < 2.0) {
        if ( !defined($regex) ) {
            croak "Can't parse old format header, termination regex needed.";
        }
        ($header, $line) = parse_old_header($infile, $header, $regex);
    } else {
        ($header, $line) = parse_new_header($infile, $header);
    }
    return ($header, $line);
}

#
#
#
#

sub parse_new_header ($$) {
    my ($infile, $header) = @_;
    my ($value, $name);
    my $line;

## Unstructured header
    while (<$infile>) {
## A line with only `#' indicates the end of this section
        if (/^#$/) {
            last;
        } else {
            ${$header}{"unstructured"} .= $_;
        }
    }

## Structured header
    while (<$infile>) {
        $line = $_; 
        warn "Read line: $line" if $debug;
## A line with only `#' indicates the end of this section
        if (/^#$/) {
            last;
        } else {
            ($value, $name) = $line =~ /^(\d+(?:\.\d+)?)\s#\s(.*)\t/;
            ${$header}{$name} += $value;
            warn "Found $name = $value\n" if $debug;
        }
    }
    return ($header, $line);
}

#
#
#
#

sub parse_old_header ($$$) {
    my ($infile, $header, $regex) = @_;
    my $line;

    while (<$infile>) {
        $line = $_;
        warn "Read line: $line" if $debug;
        if (/$regex/) {
            last;
        } else {
            $header = old_parse_header_line($line,$header);
        }
    }
    return ($header, $line);
}

#
#
#
#

sub old_parse_header_line ($$) {
    my ($line, $header) = @_;
    warn "Processing header line $line" if $debug > 7;

## CVS lines
    if ( $line =~ /^#\$/) {
        warn "Adding CVS line to header: $line" if $debug;
        ${$header}{"unstructured"} .= $line;

## Trace filenames
    } elsif ( $line =~ /^#Trace File:/ ) {
        ${$header}{"unstructured"} .= $line;
        warn "Found new trace filename, header is: ${$header}{'unstructured'}" if $debug;

## Interesting port lists
    } elsif ( $line =~ /Interesting ..P Ports/ ) {
        ${$header}{"unstructured"} .= $line; 

## Total packet and byte counts
    } elsif ( $line =~
            /^#(\d+) (?:IP )?Packets, (\d+) Total (?:IP )?Bytes(?:\.)?$/ ) {
        ${$header}{"total_packets"} += $1;
        ${$header}{"total_bytes"} += $2;
        warn "${$header}{'total_packets'} Packets, ${$header}{'total_bytes'} Total Bytes." if $debug;

## Trace duration
    } elsif ( $line =~
            /^#Starts at (\d+\.\d+), Ends at (\d+\.\d+), Duration (\d+\.\d+)/ ) {
## Put start and end times in unstructured header
         ${$header}{"unstructured"} .= $line;
#        ${$header}{"start_time"} = $1;
#        ${$header}{"end_time"} = $2;
        ${$header}{"total_time"} += $3;
        warn "Found trace duration ${$header}{'total_time'}\n" if $debug;

## IP packets with header options and Fragments of IP packets
    } elsif ( $line =~
            /^#(\d+) Optioned IP Packets, (\d+) IP Fragments/ ) {
        ${$header}{"opcount"} += $1;
        ${$header}{"fragcount"} += $2;
        warn "Found option count ${$header}{'opcount'}, fragment count ${$header}{'fragcount'}\n" if $debug;

## Fragments of TCP and UDP packets
    } elsif ( $line =~
            /^#(\d+) TCP Fragments, (\d+) UDP Fragments/ ) {
        ${$header}{"tcpfragcount"} += $1;
        ${$header}{"udpfragcount"} += $2;
        warn "Found TCP fragments = ${$header}{'tcpfragcount'}, UDP fragments = ${$header}{'udpfragcount'}\n" if $debug;

## IP packets with DF flag set
    } elsif ( $line =~ /^#(\d+) IP packets with DF set/ ) {
        ${$header}{"nofragcount"} += $1;
        warn "Found DF count ${$header}{'nofragcount'}\n" if $debug;

## Non-IP PDUs
    } elsif ( $line =~ /^#(\d+) non-IP packets/ ) {
        ${$header}{"nonip"} += $1;
        warn "Found non-IP packets = ${$header}{'nonip'}\n" if $debug;

## Non-routable Source Address
    } elsif ( $line =~ /^#(\d+) IP Packets with non-routable source addresses containing (\d+) Bytes./ ) {
        ${$header}{"packets_bogus_src"} += $1;
        ${$header}{"bytes_bogus_src"} += $2;

## Non-routable Destination Address
    } elsif ( $line =~ /^#(\d+) with non-routable destination addresses containing (\d+) Bytes./ ) {
        ${$header}{"packets_bogus_dst"} += $1;
        ${$header}{"bytes_bogus_dst"} += $2;

    }
    warn "Done with line.\n" if $debug > 7;
    return ($header);
}

#
#
# Subroutine: parse_plen_hist()
#
#

sub parse_plen_hist ($$) {
    my ($infile, $histogram) = @_;
    my ($plen, $packets);
    my $line;

    while (<$infile>) {

## A comment line indicates the end of the table
	if (/^#/) {
            $line  = $_;
	    last;
	} else {
	    ($plen, $packets) = /^(\d+)\s(\d+)/;
	    printf( STDERR "%.f\t%.f\n", $plen, $packets) if $debug > 3;
            $histogram = inc_counts($histogram,$plen,[$packets]);
        }
    }
    return ($histogram, $line);
}

#
#
# Subroutine: parse_proto_table()
#
# Description: This routine accepts a filehandle and parses a std.
#              crl_hist 6-column protocol table out of it.  It returns
#              a 2-D list of the table elements
#

sub parse_proto_table ($$) {
    my ($infile, $table) = @_;
    my ($proto, $packets, $ppct, $bytes, $bpct, $asize);
    my $line;

    while (<$infile>) {

## A comment line indicates the end of the table
	if (/^#/) {
            $line = $_;
	    last;
	} else {
	    ($proto, $packets, $ppct, $bytes, $bpct, $asize) =
		/^(\d+)\s(\d+)\s(\d+\.\d+)\s(\d+)\s(\d+\.\d+)\s(\d+)/;
	    printf( STDERR "%s\t%s\t%.f\t%2.4f\t%.f\t%2.4f\n", $proto,
		$packets,
#                $packets/$total_packets*100,
                $bytes,
#                $bytes/$total_bytes*100,
#                $asize
                ) if $debug > 3;

            $table = inc_counts($table,$proto,[$packets, $bytes]);
        }
    }
    return ($table, $line);
}

#
#
# Subroutine: parse_port_table()
#
# Description: This routine accepts a filehandle and parses a std.
#              crl_hist 7-column port table out of it.  It returns
#              a hash of lists of the table elements
#

sub parse_port_table ($$) {
    my ($infile, $table) = @_;
    my ($src, $dst, $packets, $ppct, $bytes, $bpct, $asize);
    my ($key, $line);

    while (<$infile>) {

## A comment line indicates the end of the table
        if (/^#/) {
            $line = $_;
            last;
        } else {
            ($src, $dst, $packets, $ppct, $bytes, $bpct, $asize) =
                /^(\d+)\s(\d+)\s(\d+)\s(\d+\.\d+)\s(\d+)\s(\d+\.\d+)\s(\d+)/;
            printf( STDERR "%s\t%s\t%.f\t%2.4f\t%.f\t%2.4f\n", $src, $dst,
                $packets,
#               $packets/$total_packets*100,
                $bytes,
#               $bytes/$total_bytes*100,
                $asize
                ) if $debug > 3;

            $key = "$src\t$dst";
            $table = inc_counts($table,$key,[$packets, $bytes]);
        }
    }
    return ($table, $line);
}

#
#
# Subroutine: parse_4_table()
#
# Description: This routine accepts a filehandle and parses a std.
#              crl_hist 6-column protocol table out of it.  It returns
#              a 2-D list of the table elements
#
#              parse_proto_table by another name...
#

sub parse_4_table ($$) {
    my ($infile, $table) = @_;
    my ($proto, $packets, $ppct, $bytes, $bpct, $asize);
    my $line;

    while (<$infile>) {

## A comment line indicates the end of the table
        if (/^#/) {
            $line = $_;
            last;
        } else {
            ($proto, $packets, $ppct, $bytes, $bpct, $asize) =
                /^(\d+)\s(\d+)\s(\d+\.\d+)\s(\d+)\s(\d+\.\d+)\s(\d+)/;
            printf( STDERR "%s\t%s\t%.f\t%2.4f\t%.f\t%2.4f\n", $proto,
                $packets,
#                $packets/$total_packets*100,
                $bytes,
#                $bytes/$total_bytes*100,
#                $asize
                ) if $debug > 3;

            $table = inc_counts($table,$proto,[$packets, $bytes]);
        }
    }
    return ($table, $line);
}

#
#
# Subroutine: parse_matrix()
#
# Description: This routine accepts a filehandle and parses a std.
#              crl_bycountry.pl 6-column table out of it.  It returns
#              a 2-D hash of the table elements, as well as a 1-D hash
#              of the largest elements for each unique source or
#              destination
#

sub parse_matrix ($$;$$) {
    my ($infile, $matrix, $by_x, $by_y) = @_;
    my ($x, $y, $packets, $ppct, $bytes, $bpct);
    my ($key, $line);

    while (<$infile>) {

## A comment line indicates the end of the table
        if (/^#/) {
            $line = $_;
            last;
        } else {
            ($x, $y, $packets, $ppct, $bytes, $bpct) =
                /^(\S+)\s(\S+)\s(\d+)\s(\d+\.\d+)\s(\d+)\s(\d+\.\d+)/;
            printf( STDERR "%s\t%s\t%.f\t%2.4f\t%.f\t%2.4f\n", $x, $y,
		$packets,
#		$packets/$total_packets*100,
		$bytes,
#		$bytes/$total_bytes*100
		) if $debug;

            $key = "$x\t$y";
            warn "key is $key\n" if $debug;
            $matrix = inc_counts($matrix,$key,[$packets, $bytes]);

## These two methods don't work to reproduce David's graphs
#            $by_x{$x} = [$packets, $bytes];
#            $by_y{$y} = [$packets, $bytes];

#            $by_xy{$x}[0] += $packets;
#            $by_xy{$x}[1] += $bytes;
#            $by_xy{$y}[0] += $packets;
#            $by_xy{$y}[1] += $bytes;
##

            if (!defined(${$by_x}{$x})) {
                ${$by_x}{$x} = [$packets, $bytes];
            } elsif (!defined(${$by_y}{$y})) {
                ${$by_y}{$y} = [$packets, $bytes];
            } else {
                if (${$by_x}{$x}[0] < $packets) { ${$by_x}{$x}[0] = $packets; }
                if (${$by_x}{$x}[1] < $bytes)   { ${$by_x}{$x}[1] = $bytes; }
                if (${$by_y}{$y}[0] < $packets) { ${$by_y}{$y}[0] = $packets; }
                if (${$by_y}{$y}[1] < $bytes)   { ${$by_y}{$y}[1] = $bytes; }
            }
        }
    }
    return ($matrix, $line, $by_x, $by_y);
}

#
#
# Subroutine: parse_5_hist()
#
#
 
sub parse_5_hist ($$) {
    my ($infile, $histogram) = @_;
    my ($key, $packets, $ppct, $bytes, $bpct);
    my $line;
 
    while (<$infile>) {
 
## A comment line indicates the end of the table
        if (/^#/) {
            $line = $_;
            last;
        } else {
            ($key, $packets, $ppct, $bytes, $bpct) =
		/^(\d+)\s(\d+)\s(\d+\.\d+)\s(\d+)\s(\d+\.\d+)/;
            printf( STDERR "%.f\t%.f\n", $key, $packets) if $debug > 3;
            $histogram = inc_counts($histogram,$key,[$packets, $bytes]);
        }
    }
    return ($histogram, $line);
}

##===========================================================================

my %header_desc = (
    "unstructured"	=> "",
    "interesting_tcp_ports" => "Interesting TCP Ports",
    "interesting_udp_ports" => "Interesting UDP Ports",
    "start_time"	=> "Earliest timestamp",
    "end_time"		=> "Latest timestamp",
    "total_time"	=> "Duration of trace(s)",
    "total_packets"	=> "Total IP Packets",
    "total_bytes"	=> "Total IP Bytes",
    "opcount"		=> "IP Packets with options",
    "fragcount"		=> "Fragments of IP Datagrams",
    "tcpfragcount"	=> "Fragments of TCP Datagrams",
    "udpfragcount"	=> "Fragments of UDP Datagrams",
    "nofragcount"	=> "IP packets with DF set",
    "nonip"		=> "Non-IP packets",
    "packets_bogus_src"	=> "Packets with non-routable source addresses",
    "packets_bogus_dst"	=> "Packets with non-routable destination addresses",
    "bytes_bogus_src"	=> "Bytes with non-routable source addresses",
    "bytes_bogus_dst"	=> "Bytes with non-routable destination addresses"
);
        
my %header_fmt = (
    "unstructured"      => "", # Suppress printing of unstructured header
    "interesting_tcp_ports" => "%s\t# %s\t= %s\n",
    "interesting_udp_ports" => "%s\t# %s\t= %s\n",
    "start_time"	=> "%.10f\t# %s\t= %s\n",
    "end_time"		=> "%.10f\t# %s\t= %s\n",
    "total_time"	=> "%.10f\t# %s\t= %s\n",
    "total_packets"	=> "%.f\t# %s\t= %s\n",
    "total_bytes"	=> "%.f\t# %s\t= %s\n",
    "opcount"		=> "%.f\t# %s\t= %s\n",
    "fragcount"		=> "%.f\t# %s\t= %s\n",
    "tcpfragcount"	=> "%.f\t# %s\t= %s\n",
    "udpfragcount"	=> "%.f\t# %s\t= %s\n",
    "nofragcount"	=> "%.f\t# %s\t= %s\n",
    "nonip"		=> "%.f\t# %s\t= %s\n",
    "packets_bogus_src"	=> "%.f\t# %s\t= %s\n",
    "packets_bogus_dst"	=> "%.f\t# %s\t= %s\n",
    "bytes_bogus_src"	=> "%.f\t# %s\t= %s\n",
    "bytes_bogus_dst"	=> "%.f\t# %s\t= %s\n"
);
        
sub print_header (%) {
    my (%header) = @_;

## Unstructured header
    print "$header{'unstructured'}";
    print "\#\n";

## Structured header
    foreach my $name ( sort keys %header ) {
        printf($header_fmt{$name},
            $header{$name}, $name, $header_desc{$name});
    }
    print "\#\n";
}

sub print_plen_hist ($@) {
    my ($histogram, $ptmp, $btmp) = @_;

    my ($packets, $cum_packets, $cum_bytes);
    $ptmp /= 100;
    $btmp /= 100;

    foreach my $i (get_keys($histogram)) {
        ($packets) = get_counts($histogram,$i);
        if ((! defined $packets) || ($packets == 0)) { next }
        $cum_packets += $packets;
        $cum_bytes += $packets*$i;
        printf("%.f\t%.f\t%.f\t%2.8f\t%.f\t%2.8f\n", $i,
                $packets,
                $cum_packets, $cum_packets/$ptmp,
                $cum_bytes, $cum_bytes/$btmp);
    }
    print "\#\n";
}

sub print_proto_table ($%) {
    my ($counts_byproto, %header) = @_;

    my ($packets, $bytes);
    my ($ptmp, $btmp);

    $ptmp = $header{"total_packets"}/100;
    $btmp = $header{"total_bytes"}/100;

    $localref = $counts_byproto;	# For by_bytes function
    foreach my $proto (sort by_bytes get_keys($counts_byproto)) {
        ($packets, $bytes) = get_counts($counts_byproto, $proto);
        my $name = "";
        printf("%u\t%.f\t%2.8f\t%.f\t%2.8f\t%u%s\n",
                $proto,
                $packets, $packets/$ptmp,
                $bytes, $bytes/$btmp,
                $bytes/$packets,
                $name);

    }
    print "\#\n";
}

sub print_port_table ($$@) {
    my ($table, $table_header, $ptmp, $btmp) = @_;

    my ($packets, $bytes);

    $ptmp /= 100;
    $btmp /= 100;

    print $table_header;

    $localref = $table;			# For by_bytes function
    foreach my $pair (sort by_bytes get_keys($table)) {
        ($packets, $bytes) = get_counts($table, $pair);
        printf("%s\t%.f\t%2.8f\t%.f\t%2.8f\t%u\n", $pair,
                $packets, $packets/$ptmp,
                $bytes, $bytes/$btmp,
                $bytes/$packets);
    }
    print "\#\n";
}

sub print_matrix ($$%) {
    my ($matrix, $table_header, %header) = @_;

    my ($packets, $bytes);
    my ($ptmp, $btmp);
    $ptmp = $header{total_packets}/100;
    $btmp = $header{total_bytes}/100;

    print $table_header;

    foreach my $pair (sort by_pairs get_keys($matrix)) {
        warn "key = $pair" if $debug;
        ($packets, $bytes) = get_counts($matrix, $pair);
        printf("%s\t%.f\t%2.8f\t%.f\t%2.8f\n", $pair,
                $packets, $packets/$ptmp,
                $bytes, $bytes/$btmp);
    }
    print "\#\n";
}

sub print_4_table ($@) {
    my ($table, $ptmp, $btmp) = @_;

    my ($packets, $bytes);

    $ptmp /= 100;
    $btmp /= 100;

    $localref = $table;			# For by_bytes function
    foreach my $pair (sort by_bytes get_keys($table)) {
        ($packets, $bytes) = get_counts($table, $pair);
        printf("%s\t%.f\t%2.8f\t%.f\t%2.8f\t%u\n", $pair,
                $packets, $packets/$ptmp,
                $bytes, $bytes/$btmp,
                $bytes/$packets);
    }
    print "\#\n";
}

sub print_4_table_top10 ($@) {
    my ($table, $ptmp, $btmp) = @_;

    my ($packets, $bytes, $count);

    $ptmp /= 100;
    $btmp /= 100;
    $count = 1;

    $localref = $table;			# For by_bytes function
    foreach my $pair (sort by_bytes get_keys($table)) {
        ($packets, $bytes) = get_counts($table, $pair);
        printf("%s\t%.f\t%2.8f\t%.f\t%2.8f\t%u\n", $pair,
                $packets, $packets/$ptmp,
                $bytes, $bytes/$btmp,
                $bytes/$packets);
        last if $count++ >= 10;
    }
    print "\#\n";
}

sub print_5_hist ($$%) {
    my ($histogram, $table_header, %header) = @_;
 
    my ($cum_packets, $cum_bytes);
    my ($ptmp, $btmp);
    $ptmp = $header{total_packets}/100;
    $btmp = $header{total_bytes}/100;
 
    print $table_header;
    foreach my $i (get_keys($histogram)) {
        ($cum_packets, $cum_bytes) = get_counts($histogram, $i);
        printf("%.f\t%.f\t%2.8f\t%.f\t%2.8f\n", $i,
                $cum_packets, $cum_packets/$ptmp,
                $cum_bytes, $cum_bytes/$btmp);
    }
    print "\#\n";
}

##===========================================================================

my %proto2name = (
	0 => 'HOPOPT',
	1 => 'ICMP',
	2 => 'IGMP',
	4 => 'IP-ENCAP',
	6 => 'TCP',
	8 => 'EGP',
	17 => 'UDP',
	29 => 'ISO-TP4',
	36 => 'XTP',
	41 => 'IPv6',
	46 => 'RSVP',
	47 => 'GRE',
	49 => 'BNA',
	50 => 'IPSEC-ESP',
	51 => 'IPSEC-AH',
	54 => 'NHRP',
	55 => 'MOBILE',
	57 => 'SKIP',
	74 => 'WSN',
	83 => 'VINES',
	88 => 'EIGRP',
	89 => 'OSPFIGP',
	93 => 'AX.25',
	94 => 'IPIP',
	96 => 'SCC-SP'
	);

my %tcp_port2name = (
	20 => 'FTP data',
	21 => 'FTP Control',
	22 => 'SSH',
	23 => 'Telnet',
	25 => 'SMTP',
	80 => 'HTTP',
	110 => 'POP Mail',
	119 => 'NNTP',
	139 => 'NetBIOS SSN',
	443 => 'HTTPS',
	554 => 'RTSP',
	3128 => 'Web Cache',
	5501 => 'Hotline',
	7070 => 'Real Audio'
	);

my %udp_port2name = (
	7 => 'Echo',
	19 => 'Chargen',
	53 => 'DNS',
	113 => 'Ident',
	123 => 'NTP',
	137 => 'NetBIOS NS',
	138 => 'NetBIOS DGM',
	6112 => 'Starcraft',
	7070 => 'Real Audio',
	7648 => 'CU-SeeMe',
	27001 => 'QuakeWorld',
	27500 => 'QuakeWorld',
	27901 => 'Quake 2',
	27910 => 'Quake 2'
	);

sub print_html_header (%) {
    my (%header) = @_;

    print "<table BORDER=1>\n";
    foreach my $name ( sort keys %header ) {
## Only print structured header
        if ( $name ne 'unstructured' )  {
            printf("<tr><td>%s</td><td>%.f</td></tr>\n",
                $header_desc{$name}, $header{$name});
        }
    }
    print "</table>\n";
}

sub print_html_proto_table ($%) {
    my ($counts_byproto, %header) = @_;

    my ($name, $packets, $bytes);
    my ($ptmp, $btmp);

    $ptmp = $header{"total_packets"}/100;
    $btmp = $header{"total_bytes"}/100;

    print "<h4>Traffic breakdown by protocol:</h4>\n";
    print "<table BORDER=1>\n";
    print "<tr><td>Protocol</td><td>Number</td><td>Packets</td><td>\% of total</td><td>Bytes</td><td>\% of total</td><td>Avg. Packet Size</td></tr>\n";

    $localref = $counts_byproto;        # For by_bytes function
    foreach my $proto (sort by_bytes get_keys($counts_byproto)) {
        ($packets, $bytes) = get_counts($counts_byproto, $proto);

        $name = $proto2name{$proto};
        if (!defined $name) { $name = ""; }

        printf("<tr><td>%s</td><td>%u</td><td>%.f</td><td>%2.4f</td><td>%.f</td><td>%2.4f</td><td>%u</td></tr>\n",
                $name, $proto,
                $packets, $packets/$ptmp,
                $bytes, $bytes/$btmp,
                $bytes/$packets
                );
    }
    print "</table>\n";
}

sub print_html_tcp_port_table ($$) {
    my ($table, $counts_byproto) = @_;

    my ($name, $packets, $bytes, $src, $dst);
    my ($top_n, $count);
    my ($ptmp, $btmp);

    $top_n = 25;

    ($ptmp, $btmp) = get_counts($counts_byproto, 6);
    $ptmp /= 100;
    $btmp /= 100;

    print "<h4>TCP traffic matrix:</h4>\n";
    print "<table BORDER=1>\n";
    print "<tr><td>Protocol</td><td>Src</td><td>Dst</td><td>Packets</td><td>\% of total</td><td>Bytes</td><td>\% of total</td><td>Avg. Packet Size</td></tr>\n";

    $localref = $table;                 # For by_bytes function
    foreach my $pair (sort by_bytes get_keys($table)) {
        ($packets, $bytes) = get_counts($table, $pair);

        ($src, $dst) = split /\t/, $pair;
        $name = $tcp_port2name{$src};
        if (!defined $name) { $name = $tcp_port2name{$dst}; }
        if (!defined $name) { $name = ""; }

        printf("<tr><td>%s</td><td>%u</td><td>%u</td><td>%.f</td><td>%2.4f</td><td>%.f</td><td>%2.4f</td><td>%u</td></tr>\n",
                $name, $src, $dst,
                $packets, $packets/$ptmp,
                $bytes, $bytes/$btmp,
                $bytes/$packets
                );

        $count++;
        last if $count >= $top_n;
    }
    print "</table>\n";
}

sub print_html_udp_port_table ($$) {
    my ($table, $counts_byproto) = @_;

    my ($name, $packets, $bytes, $src, $dst);
    my ($top_n, $count);
    my ($ptmp, $btmp);

    $top_n = 25;

    ($ptmp, $btmp) = get_counts($counts_byproto, 17);
    $ptmp /= 100;
    $btmp /= 100;

    print "<h4>UDP traffic matrix:</h4>\n";
    print "<table BORDER=1>\n";
    print "<tr><td>Protocol</td><td>Src</td><td>Dst</td><td>Packets</td><td>\% of total</td><td>Bytes</td><td>\% of total</td><td>Avg. Packet Size</td></tr>\n";

    $localref = $table;                 # For by_bytes function
    foreach my $pair (sort by_bytes get_keys($table)) {
        ($packets, $bytes) = get_counts($table, $pair);

        ($src, $dst) = split /\t/, $pair;
        $name = $udp_port2name{$src};
        if (!defined $name) { $name = $udp_port2name{$dst}; }
        if (!defined $name) { $name = ""; }

        printf("<tr><td>%s</td><td>%u</td><td>%u</td><td>%.f</td><td>%2.4f</td><td>%.f</td><td>%2.4f</td><td>%u</td></tr>\n",
                $name, $src, $dst,
                $packets, $packets/$ptmp,
                $bytes, $bytes/$btmp,
                $bytes/$packets
                );

        $count++;
        last if ( $count >= $top_n );
    }
    print "</table>\n";
}

sub print_html_matrix ($$%) {
    my ($matrix, $table_header, %header) = @_;

    my ($packets, $bytes, $src, $dst);
    my ($top_n, $count);
    my ($ptmp, $btmp);

    $top_n = 30;

    $ptmp = $header{total_packets}/100;
    $btmp = $header{total_bytes}/100;

    print $table_header;
    print "<table BORDER=1>\n";
    print "<tr><td>Src</td><td>Dst</td><td>Packets</td><td>\% of total</td><td>Bytes</td><td>\% of total</td></tr>\n";

    $localref = $matrix;                 # For by_bytes function
    foreach my $pair (sort by_bytes get_keys($matrix)) {
        warn "key = $pair" if $debug;
        ($packets, $bytes) = get_counts($matrix, $pair);
        ($src, $dst) = split /\t/, $pair;
        printf("<tr><td>%.f</td><td>%.f</td><td>%.f</td><td>%2.8f</td><td>%.f</td><td>%2.8f</td></tr>\n",
		$src, $dst,
                $packets, $packets/$ptmp,
                $bytes, $bytes/$btmp);

        $count++;
        last if $count >= $top_n;
    }
    print "</table>\n";
}

sub print_html_5_hist ($$%) {
    my ($histogram, $table_header, %header) = @_;

    my ($cum_packets, $cum_bytes);
    my ($ptmp, $btmp);
    $ptmp = $header{total_packets}/100;
    $btmp = $header{total_bytes}/100;

    print $table_header;

    print "<table BORDER=1>\n";
    foreach my $i (get_keys($histogram)) {
        ($cum_packets, $cum_bytes) = get_counts($histogram, $i);
        printf("<tr><td>%.f</td><td>%.f</td><td>%2.8f</td><td>%.f</td><td>%2.8f</td></tr>\n",
                $i,
                $cum_packets, $cum_packets/$ptmp,
                $cum_bytes, $cum_bytes/$btmp
                );
    }
    print "</table>\n";
}

