#ifndef IPV6ADDR_HH
#define IPV6ADDR_HH

extern "C" {
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
}

#include <exception>
#include <stdexcept>

class ipv6addr_t : private in6_addr {
public:
    static const int family = AF_INET6;
    ipv6addr_t() { memset(this, 0, sizeof(*this)); }
    ipv6addr_t(const in6_addr &src)
	{ *this = *static_cast<const ipv6addr_t*>(&src); }
    ipv6addr_t(const char *str) { parse(str); }
    void parse(const char *str) {
	if (inet_pton(AF_INET6, str, this) != 1) {
	    throw std::runtime_error(std::string("can't parse IPv6 address ") + str);
	}
    }
    ipv6addr_t operator= (const in6_addr &src)
	{ return (*this = *static_cast<const ipv6addr_t*>(&src)); }

    bool operator < (const ipv6addr_t &b) const
    {
	for (size_t i = 0; i < sizeof(s6_addr); i++)
	    if (((uint8_t*)this)[i] != ((uint8_t*)&b)[i])
		return ((uint8_t*)this)[i] < ((uint8_t*)&b)[i];
	return false;
    }

    bool operator == (const ipv6addr_t &b) const
    {
	for (size_t i = 0; i < sizeof(s6_addr); i++)
	    if (((uint8_t*)this)[i] != ((uint8_t*)&b)[i])
		return false;
	return true;
    }

    ipv6addr_t operator & (const ipv6addr_t &b) const; // not implemented
    ipv6addr_t operator | (const ipv6addr_t &b) const; // not implemented
    ipv6addr_t operator ^ (const ipv6addr_t &b) const; // not implemented

    ipv6addr_t & operator &= (const ipv6addr_t &b); // not implemented
    ipv6addr_t & operator |= (const ipv6addr_t &b); // not implemented
    ipv6addr_t & operator ^= (const ipv6addr_t &b); // not implemented

    bool bittest(size_t k) const {
	return ((const uint8_t*)(this))[k/8] & (1 << ((127-k)%8));
    }

    friend std::ostream & operator << (std::ostream & os, const ipv6addr_t &a)
    {
	char buf[64];
	return os << inet_ntop(AF_INET6, &a, buf, sizeof(buf));
    }
};

#endif // IPV6ADDR_HH
