// interactive test tool for IpPrefixPatricia
// usage:  testpatricia {4|4h|6}
// commands:
//    insert prefix:             p{pfx} {text}
//    lookup address:            a{addr}
//    find ALL matches of addr:  A{addr}
//    erase from pfx1 to pfx2:   e{pfx1} {pfx2}
//    erase from pfx1 to end:    E{pfx1}
//    clear all prefixes:        c
//    dump prefix table:         d
//    dump a single prefix:      d{pfx}
//    addr1 > addr2:             >{addr1} {addr2}
//    addr1 < addr2:             <{addr1} {addr2}

extern "C" {
#include <stdio.h>
#include <sys/types.h>
#include <netdb.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>
#include <syslog.h>
#include <fcntl.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <errno.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <time.h>

    extern int errno;
}

#include <string>
#include <map>

#include "Ipv4Network.hh"
#include "Ipv6Network.hh"
#include "IpPrefixPatricia.hh"

using namespace std;

IpPrefixPatricia<Ipv4Network, char *> pfxmap4;
IpPrefixPatricia<Ipv4hNetwork, char *> pfxmap4h;
IpPrefixPatricia<Ipv6Network, char *> pfxmap6;

using namespace std;

template<class IpNetwork>
void dump_prefixes(IpPrefixPatricia<IpNetwork, char *> & pfxmap)
{
    typename IpPrefixPatricia<IpNetwork, char *>::const_iterator it;
    cerr << "# Prefixes:" << endl;
    for (it = pfxmap.begin(); it != pfxmap.end(); it++) {
	cerr << (*it).first << '\t' << (*it).second << endl;
    }
}

#define die(str) \
    do { cerr << str << ", line " << line << endl; exit(1); } while (0)

template<class IpNetwork>
void stuff(FILE *file, IpPrefixPatricia<IpNetwork, char *>pfxmap)
{
    typename IpNetwork::ipaddr_type addr, addr2;
    int line;
    char buf[80];
    char addrstr[16], addrstr2[16], str[81];
    int len, len2;
    typename IpPrefixPatricia<IpNetwork, char *>::const_iterator cit;
    typename IpPrefixPatricia<IpNetwork, char *>::iterator it, it2;
    typename IpPrefixPatricia<IpNetwork, char *>::const_match_iterator mit;

    line = 1;
    while (fgets(buf, sizeof(buf), file)) {
	char *p = buf+1;
	while (isspace(*p)) p++;
	buf[strlen(buf)-1] = '\0';
	switch (buf[0]) {
	case '#':
	case '\0':
	    break;

	case 'p':
	    if (sscanf(p, "%15[a-fA-F0-9.:]/%d %80[^\n]", addrstr, &len, str) != 3) {
		cerr << "syntax error" << endl;
		continue;
	    }
	    {
		addr.parse(addrstr);
		IpNetwork net(addr, len);
		cerr << "prefix: " << net << endl;
		pfxmap[net] = strdup(str);
	    }
	    break;

	case 'a':
	    addr.parse(p);
	    cit = pfxmap.LongestMatch(addr);
	    cerr << addr << '\t';
	    if (cit == pfxmap.end()) {
		cerr << "NO MATCH" << endl;
	    } else {
		cerr << (*cit).first << '\t' << (*cit).second << endl;
	    }
	    break;

	case 'A':
	    addr.parse(p);
	    mit = pfxmap.ShortestMatchIt(addr);
	    if (mit == pfxmap.endMatchIt()) {
		cerr << addr << '\t';
		cerr << "NO MATCH" << endl;
	    } else {
		while (mit != pfxmap.endMatchIt()) {
		    cerr << addr << '\t';
		    cerr << (*mit).first << '\t' << (*mit).second << endl;
		    ++mit;
		}
	    }
	    break;

	case 'd':
	    if (!*p) {
		dump_prefixes(pfxmap);
	    } else if (sscanf(p, "%15[a-fA-F0-9.:]/%d", addrstr, &len) == 2) {
		addr.parse(addrstr);
		IpNetwork net(addr, len);
		cerr << net << " " << pfxmap[net] << endl;
	    } else {
		cerr << "syntax error" << endl;
		continue;
	    }
	    break;

	case 'e':
	    if (sscanf(p, "%15[a-fA-F0-9.:]/%d", addrstr, &len) != 2) {
		cerr << "syntax error" << endl;
		continue;
	    }
	    {
		addr.parse(addrstr);
		IpNetwork net(addr, len);
		cerr << "erase " << net << ":";
		it = pfxmap.find(net);
		if (it == pfxmap.end()) {
		    cerr << "NO MATCH" << endl;
		} else {
		    pfxmap.erase(it);
		    cerr << "ok" << endl;
		}
	    }
	    break;

	case 'R':
	    if (sscanf(p, "%15[a-fA-F0-9.:]/%d %15[a-fA-F0-9.:]/%d",
		    addrstr, &len, addrstr2, &len2) != 4)
	    {
		cerr << "syntax error" << endl;
		continue;
	    }
	    {
		addr.parse(addrstr);
		addr2.parse(addrstr2);
		IpNetwork net(addr, len);
		IpNetwork net2(addr2, len2);
		cerr << "erase from " << net << " to " << net2 << ": ";
		it = pfxmap.find(net);
		it2 = pfxmap.find(net2);
		if (it == pfxmap.end() || it2 == pfxmap.end()) {
		    cerr << "NO MATCH" << endl;
		} else {
		    pfxmap.erase(it, it2);
		    cerr << "ok" << endl;
		}
	    }
	    break;

        case 'E':
            if (sscanf(p, "%15[a-fA-F0-9.:]/%d", addrstr, &len) != 2) {
                cerr << "syntax error" << endl;
		continue;
	    }
            {
		addr.parse(addrstr);
                IpNetwork net(addr, len);
		cerr << "erase from " << net << " to end: ";
                it = pfxmap.find(net);
                if (it == pfxmap.end()) {
		    cerr << "NO MATCH" << endl;
                } else {
                    pfxmap.erase(it, pfxmap.end());
		    cerr << "ok" << endl;
                }
            }
            break;

	case 'c':
	    cerr << "clear" << endl;
	    pfxmap.clear();
	    break;

	case '>':
	    if (sscanf(p, "%15[a-fA-F0-9.:] %15[a-fA-F0-9.:]",
		    addrstr, addrstr2) != 2)
	    {
		cerr << "syntax error" << endl;
		continue;
	    }
	    addr.parse(addrstr);
	    addr2.parse(addrstr2);
	    cerr << addr << " > " << addr2 << "? " << (addr2 < addr) << endl;
	    break;

	case '<':
	    if (sscanf(p, "%15[a-fA-F0-9.:] %15[a-fA-F0-9.:]",
		    addrstr, addrstr2) != 2)
	    {
		cerr << "syntax error" << endl;
		continue;
	    }
	    addr.parse(addrstr);
	    addr2.parse(addrstr2);
	    cerr << addr << " < " << addr2 << "? " << (addr < addr2) << endl;
	    break;

	default:
	    cerr << "unknown command '" << buf[0] << "'" << endl;
	    break;
	}
	line++;
    }
}

int main(int argc, char *argv[])
{
    if (argc == 2 && strcmp(argv[1], "4") == 0)
	stuff(stdin, pfxmap4);
    else if (argc == 2 && strcmp(argv[1], "4h") == 0)
	stuff(stdin, pfxmap4h);
    else if (argc == 2 && strcmp(argv[1], "6") == 0)
	stuff(stdin, pfxmap6);
    else {
	cerr << "usage: " << argv[0] << " {4|4h|6}" << endl;
	exit(1);
    }
    return 0;
}
