##
## Copyright 1999, 2000, 2001, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2011, 2012, 2019 The Regents of the University of California
## All Rights Reserved 
##
## Permission to use, copy, modify and distribute any part of this
## CoralReef software package for educational, research and non-profit
## purposes, without fee, and without a written agreement is hereby
## granted, provided that the above copyright notice, this paragraph
## and the following paragraphs appear in all copies.
##
## Those desiring to incorporate this into commercial products or use
## for commercial purposes should contact the Technology Transfer
## Office, University of California, San Diego, 9500 Gilman Drive, La
## Jolla, CA 92093-0910, Ph: (858) 534-5815, FAX: (858) 534-7345.
##
## IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
## PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
## DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
## SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
## THE POSSIBILITY OF SUCH DAMAGE.
##
## THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
## UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
## SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
## OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
## OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
## TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
## PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
## ANY PATENT, TRADEMARK OR OTHER RIGHTS.
##
## The CoralReef software package is developed by the CoralReef
## development team at the University of California, San Diego under
## the Cooperative Association for Internet Data Analysis (CAIDA)
## Program. Support for this effort is provided by the CAIDA grant
## NCR-9711092, DARPA NGI Contract N66001-98-2-8922, DARPA NMS Grant
## N66001-01-1-8909, and by CAIDA members.
##
## Report bugs and suggestions to coral-bugs@caida.org.
##
## ---------------------------------------------------------------------
## Perl module:  CAIDA::Tables::IP_Table.pm

package CAIDA::Tables::IP_Table;

# Set up object export condition.  No functions accessible directly.
use CAIDA::Tables::Generic;

# Use externally defined functions.
use CAIDA::Tables::make_Country_Table; 

# Inherit from the pack based generic table code.
@ISA = (CAIDA::Tables::Generic::SingleKey);

# Version of module
$VERSION = 2.0;

# Define required CVS variables
$cvs_Id = '$Id: IP_Table.pm,v 1.31 2019/03/28 23:40:52 kkeys Exp $';
$cvs_Author = '$Author: kkeys $';
$cvs_Name = '$Name: release-3-9-5 $';
$cvs_Revision = '$Revision: 1.31 $';


# Socket::inet_aton and co
use Socket;

# Force this module to adhere to safe programming practices.
use strict;


# .   .   .   .   .  User Method Subroutines  .   .   .   .   .   .   .   .

sub _initialize (@ ) {
    my ($self, $arg_hash_ref) = @_;

    $self->{'_kind'} = "IP_Table";
    $self->{'_implode_filters'} = [ \&Socket::inet_aton ];
    $self->{'_explode_filters'} = [ \&Socket::inet_ntoa ];
    $self->{'_num_fields'} = 1;

    $self->SUPER::_initialize($arg_hash_ref);
}

sub make_AS_Table (@ ) {
    my ($self, $arg_hash_ref) = @_;
    my $as_finder = $arg_hash_ref->{'as_finder'};
    if (not $as_finder) {
	die "Need an ASFinder object to be able to do AS lookups.";
    }
    
    require CAIDA::Tables::AS_Table;
    my $as_table = new CAIDA::Tables::AS_Table();
    while (my ($opaque_key, $counts) = each %{ $self->data() }) {
	my ($dotted) = $self->get_key_fields($opaque_key);
	my ($as, $net, $masklen) = $as_finder->get_as($dotted);
	if (!defined($as)) {
	    $as = "NOROUTE";
	}
	($as) = split(/\s/, $as); # Remove any trailing information
	$as_table->entry_add($as, $counts);
    }
    $as_table->total->add($self->other);
    $as_table->other->add($self->other);
    return $as_table;
}   


# .   .   .   .   .   .   .   .   .   .   .   .   .   .   .   .   .   .   . 
1; ##must be last line in the file
