##
## Copyright 1999, 2000, 2001, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2011, 2012, 2019 The Regents of the University of California
## All Rights Reserved 
##
## Permission to use, copy, modify and distribute any part of this
## CoralReef software package for educational, research and non-profit
## purposes, without fee, and without a written agreement is hereby
## granted, provided that the above copyright notice, this paragraph
## and the following paragraphs appear in all copies.
##
## Those desiring to incorporate this into commercial products or use
## for commercial purposes should contact the Technology Transfer
## Office, University of California, San Diego, 9500 Gilman Drive, La
## Jolla, CA 92093-0910, Ph: (858) 534-5815, FAX: (858) 534-7345.
##
## IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
## PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
## DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
## SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
## THE POSSIBILITY OF SUCH DAMAGE.
##
## THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
## UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
## SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
## OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
## OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
## TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
## PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
## ANY PATENT, TRADEMARK OR OTHER RIGHTS.
##
## The CoralReef software package is developed by the CoralReef
## development team at the University of California, San Diego under
## the Cooperative Association for Internet Data Analysis (CAIDA)
## Program. Support for this effort is provided by the CAIDA grant
## NCR-9711092, DARPA NGI Contract N66001-98-2-8922, DARPA NMS Grant
## N66001-01-1-8909, and by CAIDA members.
##
## Report bugs and suggestions to coral-bugs@caida.org.
##
## ---------------------------------------------------------------------
## Perl module:  CAIDA::Tables::IPv6_Tuple_Table.pm

package CAIDA::Tables::IPv6_Tuple_Table;

# Set up object export condition.  No functions accessible directly.
use CAIDA::Tables::Generic;

# Use externally defined functions.
#use CAIDA::Tables::make_Country_Matrix; 
#use CAIDA::Tables::make_Country_Table; 

# Inherit from the pack based generic table code.
@ISA = (CAIDA::Tables::Generic::Pack);

# Version of module
$VERSION = 2.0;

# Define required CVS variables
$cvs_Id = '$Id: IPv6_Tuple_Table.pm,v 1.3 2019/03/28 23:40:52 kkeys Exp $';
$cvs_Author = '$Author: kkeys $';
$cvs_Name = '$Name: release-3-9-5 $';
$cvs_Revision = '$Revision: 1.3 $';

# Socket::inet_aton and co
use Socket;
use Socket6;
#qw(inet_pton inet_ntop);

# Force this module to adhere to safe programming practices.
use strict;


# .   .   .   .   .  User Method Subroutines  .   .   .   .   .   .   .   .

sub _v6_pton {
    return Socket6::inet_pton(AF_INET6, $_[0]);
}

sub _v6_ntop {
    return Socket6::inet_ntop(AF_INET6, $_[0]);
}

sub _initialize (@ ) {
    my ($self, $arg_hash_ref) = @_;

    $self->{'_kind'} = "IPv6_Tuple_Table";
    $self->{'_pack_elements'} = [ "a16", "a16", "C", "C", "n", "n" ];
    $self->{'_implode_filters'} = [ \&_v6_pton,
				    \&_v6_pton,
				    undef,
				    undef,
				    undef,
				    undef,
				  ];
    $self->{'_explode_filters'} = [ \&_v6_ntop,
				    \&_v6_ntop,
				    undef,
				    undef,
				    undef,
				    undef,
				  ];
    $self->{'_num_fields'} = 6;

    $self->SUPER::_initialize($arg_hash_ref);
}

=cut
sub make_src_IP_Table (@ ) {
# -----------------------------------------------
# Quasi constructor function for source IP
# tables.  It is a hack in that it depends on 
# the internal representation of IP_tables
# being a variation on IPv6_Tuple_Table.
# -----------------------------------------------
    my ($self) = @_;
    
    my $result = $self->aggregate_columns(0);
    
    # XXX this depends on knowing that the internal representation of
    # an IP_Table is the same as aggregate_columns(0) on a IPv6_Tuple_Table.
    $result->{'_kind'} = "IP_Table";
    require CAIDA::Tables::IP_Table;
    bless $result, "CAIDA::Tables::IP_Table";
    
    return $result;
}   

sub make_dst_IP_Table (@ ) {
# -----------------------------------------------
# Quasi constructor function for destination IP
# tables.  It is a hack in that it depends on 
# the internal representation of IP_tables
# being a variation on Tuple_Table.
# -----------------------------------------------
    my ($self) = @_;
    
    my $result = $self->aggregate_columns(1);
    
    # XXX this depends on knowing that the internal representation of
    # an IP_Table is the same as aggregate_columns(1) on a IPv6_Tuple_Table.
    $result->{'_kind'} = "IP_Table";
    require CAIDA::Tables::IP_Table;
    bless $result, "CAIDA::Tables::IP_Table";
    
    return $result;
}   
=cut

sub make_src_Port_Table (@ ) {
    my ($self, $arg_hash_ref) = @_;
    my $protocol = $arg_hash_ref->{'protocol'};
    
    require CAIDA::Tables::Port_Table;
    my $port_table = new CAIDA::Tables::Port_Table();
    while (my ($opaque_key, $counts) = each %{ $self->data() }) {
	my ($s_ip, $d_ip, $ip_proto, $p_ok, $src_port, $dst_port) =
				$self->get_key_fields($opaque_key);
	unless (defined $protocol and $protocol != $ip_proto) {
	    $port_table->entry_add($src_port, $counts);
	}
    }
    $port_table->total->add($self->other);
    $port_table->other->add($self->other);
    return $port_table;
}   

sub make_dst_Port_Table (@ ) {
    my ($self, $arg_hash_ref) = @_;
    my $protocol = $arg_hash_ref->{'protocol'};
    
    require CAIDA::Tables::Port_Table;
    my $port_table = new CAIDA::Tables::Port_Table();
    while (my ($opaque_key, $counts) = each %{ $self->data() }) {
	my ($s_ip, $d_ip, $ip_proto, $p_ok, $src_port, $dst_port) =
				$self->get_key_fields($opaque_key);
	unless (defined $protocol and $protocol != $ip_proto) {
	    $port_table->entry_add($dst_port, $counts);
	}
    }
    $port_table->total->add($self->other);
    $port_table->other->add($self->other);
    return $port_table;
}   

sub make_src_Proto_Port_Table (@ ) {
    my ($self, $arg_hash_ref) = @_;
    my $protocol = $arg_hash_ref->{'protocol'};
    
    require CAIDA::Tables::Proto_Port_Table;
    my $port_table = new CAIDA::Tables::Proto_Port_Table();
    while (my ($opaque_key, $counts) = each %{ $self->data() }) {
	my ($s_ip, $d_ip, $ip_proto, $p_ok, $src_port, $dst_port) =
				$self->get_key_fields($opaque_key);
	unless (defined $protocol and $protocol != $ip_proto) {
	    $port_table->entry_add($ip_proto, $p_ok, $src_port, $counts);
	}
    }
    $port_table->total->add($self->other);
    $port_table->other->add($self->other);
    return $port_table;
}   

sub make_dst_Proto_Port_Table (@ ) {
    my ($self, $arg_hash_ref) = @_;
    my $protocol = $arg_hash_ref->{'protocol'};
    
    require CAIDA::Tables::Proto_Port_Table;
    my $port_table = new CAIDA::Tables::Proto_Port_Table();
    while (my ($opaque_key, $counts) = each %{ $self->data() }) {
	my ($s_ip, $d_ip, $ip_proto, $p_ok, $src_port, $dst_port) =
				$self->get_key_fields($opaque_key);
	unless (defined $protocol and $protocol != $ip_proto) {
	    $port_table->entry_add($ip_proto, $p_ok, $dst_port, $counts);
	}
    }
    $port_table->total->add($self->other);
    $port_table->other->add($self->other);
    return $port_table;
}   


=cut
sub make_src_IP_Proto_Ports_Table (@ ) {
    my ($self, $arg_hash_ref) = @_;
    my $protocol = $arg_hash_ref->{'protocol'};
    
    require CAIDA::Tables::IP_Proto_Ports_Table;
    my $port_table = new CAIDA::Tables::IP_Proto_Ports_Table();
    while (my ($opaque_key, $counts) = each %{ $self->data() }) {
	my ($s_ip, $d_ip, $ip_proto, $p_ok, $src_port, $dst_port) =
				$self->get_key_fields($opaque_key);
	unless (defined $protocol and $protocol != $ip_proto) {
	    $port_table->entry_add($s_ip, $ip_proto, $p_ok, $src_port, $dst_port, $counts);
	}
    }
    $port_table->total->add($self->other);
    $port_table->other->add($self->other);
    return $port_table;
}   

sub make_dst_IP_Proto_Ports_Table (@ ) {
    my ($self, $arg_hash_ref) = @_;
    my $protocol = $arg_hash_ref->{'protocol'};
    
    require CAIDA::Tables::IP_Proto_Ports_Table;
    my $port_table = new CAIDA::Tables::IP_Proto_Ports_Table();
    while (my ($opaque_key, $counts) = each %{ $self->data() }) {
	my ($s_ip, $d_ip, $ip_proto, $p_ok, $src_port, $dst_port) =
				$self->get_key_fields($opaque_key);
	unless (defined $protocol and $protocol != $ip_proto) {
	    $port_table->entry_add($d_ip, $ip_proto, $p_ok, $src_port, $dst_port, $counts);
	}
    }
    $port_table->total->add($self->other);
    $port_table->other->add($self->other);
    return $port_table;
}   


sub make_src_IP_Proto_src_Port_Table (@ ) {
    my ($self, $arg_hash_ref) = @_;
    my $protocol = $arg_hash_ref->{'protocol'};
    
    require CAIDA::Tables::IP_Proto_Port_Table;
    my $port_table = new CAIDA::Tables::IP_Proto_Port_Table();
    while (my ($opaque_key, $counts) = each %{ $self->data() }) {
	my ($s_ip, $d_ip, $ip_proto, $p_ok, $src_port, $dst_port) =
				$self->get_key_fields($opaque_key);
	unless (defined $protocol and $protocol != $ip_proto) {
	    $port_table->entry_add($s_ip, $ip_proto, $p_ok, $src_port, $counts);
	}
    }
    $port_table->total->add($self->other);
    $port_table->other->add($self->other);
    return $port_table;
}   

sub make_src_IP_Proto_dst_Port_Table (@ ) {
    my ($self, $arg_hash_ref) = @_;
    my $protocol = $arg_hash_ref->{'protocol'};
    
    require CAIDA::Tables::IP_Proto_Port_Table;
    my $port_table = new CAIDA::Tables::IP_Proto_Port_Table();
    while (my ($opaque_key, $counts) = each %{ $self->data() }) {
	my ($s_ip, $d_ip, $ip_proto, $p_ok, $src_port, $dst_port) =
				$self->get_key_fields($opaque_key);
	unless (defined $protocol and $protocol != $ip_proto) {
	    $port_table->entry_add($s_ip, $ip_proto, $p_ok, $dst_port, $counts);
	}
    }
    $port_table->total->add($self->other);
    $port_table->other->add($self->other);
    return $port_table;
}   
sub make_dst_IP_Proto_src_Port_Table (@ ) {
    my ($self, $arg_hash_ref) = @_;
    my $protocol = $arg_hash_ref->{'protocol'};
    
    require CAIDA::Tables::IP_Proto_Port_Table;
    my $port_table = new CAIDA::Tables::IP_Proto_Port_Table();
    while (my ($opaque_key, $counts) = each %{ $self->data() }) {
	my ($s_ip, $d_ip, $ip_proto, $p_ok, $src_port, $dst_port) =
				$self->get_key_fields($opaque_key);
	unless (defined $protocol and $protocol != $ip_proto) {
	    $port_table->entry_add($d_ip, $ip_proto, $p_ok, $src_port, $counts);
	}
    }
    $port_table->total->add($self->other);
    $port_table->other->add($self->other);
    return $port_table;
}   

sub make_dst_IP_Proto_dst_Port_Table (@ ) {
    my ($self, $arg_hash_ref) = @_;
    my $protocol = $arg_hash_ref->{'protocol'};
    
    require CAIDA::Tables::IP_Proto_Port_Table;
    my $port_table = new CAIDA::Tables::IP_Proto_Port_Table();
    while (my ($opaque_key, $counts) = each %{ $self->data() }) {
	my ($s_ip, $d_ip, $ip_proto, $p_ok, $src_port, $dst_port) =
				$self->get_key_fields($opaque_key);
	unless (defined $protocol and $protocol != $ip_proto) {
	    $port_table->entry_add($d_ip, $ip_proto, $p_ok, $dst_port, $counts);
	}
    }
    $port_table->total->add($self->other);
    $port_table->other->add($self->other);
    return $port_table;
}   


sub make_IP_Matrix (@ ) {
# -----------------------------------------------
# Quasi constructor function for IP
# matrices.  It is a hack in that it depends on 
# the internal representation of IP_tables
# being a variation on IPv6_Tuple_Table.
# -----------------------------------------------
    my ($self) = @_;
    
    my $result = $self->aggregate_columns(0,1);
    
    # XXX this depends on knowing that the internal representation of
    # an IP_Matrix is the same as aggregate_columns(0,1) on a Tuple_Table.
    $result->{'_kind'} = "IP_Matrix";
    require CAIDA::Tables::IP_Matrix;
    bless $result, "CAIDA::Tables::IP_Matrix";
    
    return $result;
}
=cut

sub make_IPv6_Proto_Table (@ ) {
# -----------------------------------------------
# Quasi constructor function for Protocol
# tables.  It is a hack in that it depends on 
# the internal representation of Proto_Table
# being a variation on Tuple_Table.
# -----------------------------------------------
    my ($self) = @_;
    
    my $result = $self->aggregate_columns(2);
    
    # XXX this depends on knowing that the internal representation of
    # an Proto_Table is the same as aggregate_columns(2) on a Tuple_Table.
    $result->{'_kind'} = "Proto_Table";
    require CAIDA::Tables::Proto_Table;
    bless $result, "CAIDA::Tables::Proto_Table";
    
    return $result;
}   


sub make_Proto_Ports_Table (@ ) {
# -----------------------------------------------
# Quasi constructor function for Protocol
# tables.  It is a hack in that it depends on 
# the internal representation of Proto_Ports_Table
# being a variation on Tuple_Table.
# -----------------------------------------------
    my ($self) = @_;
    
    my $result = $self->aggregate_columns(2,3,4,5);
    
    # XXX this depends on knowing that the internal representation of
    # an Proto_Ports_Table is the same as
    # aggregate_columns(2,3,4,5) on a Tuple_Table.
    $result->{'_kind'} = "Proto_Ports_Table";
    require CAIDA::Tables::Proto_Ports_Table;
    bless $result, "CAIDA::Tables::Proto_Ports_Table";
    
    return $result;
}   

=cut
sub make_AS_Matrix (@ ) {
    my ($self, $arg_hash_ref) = @_;
    my $as_finder = $arg_hash_ref->{'as_finder'};
    if (not $as_finder) {
	die "Need an ASFinder object to be able to do AS lookups.";
    }
    
    require CAIDA::Tables::AS_Matrix;
    my $as_matrix = new CAIDA::Tables::AS_Matrix();
    while (my ($opaque_key, $counts) = each %{ $self->data() }) {
	my ($src_dotted, $dst_dotted) = $self->get_key_fields($opaque_key);
	my ($src_as) = $as_finder->get_as($src_dotted);
	my ($dst_as) = $as_finder->get_as($dst_dotted);
	$src_as = "NOROUTE" unless defined $src_as;
	$dst_as = "NOROUTE" unless defined $dst_as;
	($src_as) = split(/\s/, $src_as); # Remove any trailing information
	($dst_as) = split(/\s/, $dst_as); # Remove any trailing information
	$as_matrix->entry_add($src_as, $dst_as, $counts);
    }
    $as_matrix->total->add($self->other);
    $as_matrix->other->add($self->other);
    return $as_matrix;
}   

sub make_src_AS_Table (@ ) {
    my ($self, $arg_hash_ref) = @_;
    my $as_finder = $arg_hash_ref->{'as_finder'};
    if (not $as_finder) {
	die "Need an ASFinder object to be able to do AS lookups.";
    }
    
    require CAIDA::Tables::AS_Matrix;
    my $as_table = new CAIDA::Tables::AS_Matrix();
    while (my ($opaque_key, $counts) = each %{ $self->data() }) {
	my ($src_dotted) = $self->get_key_fields($opaque_key);
	my ($src_as) = $as_finder->get_as($src_dotted);
	$src_as = "NOROUTE" unless defined $src_as;
	($src_as) = split(/\s/, $src_as); # Remove any trailing information
	$as_table->entry_add($src_as, $counts);
    }
    $as_table->total->add($self->other);
    $as_table->other->add($self->other);
    return $as_table;
}

sub make_dst_AS_Table (@ ) {
    my ($self, $arg_hash_ref) = @_;
    my $as_finder = $arg_hash_ref->{'as_finder'};
    if (not $as_finder) {
	die "Need an ASFinder object to be able to do AS lookups.";
    }
    
    require CAIDA::Tables::AS_Matrix;
    my $as_table = new CAIDA::Tables::AS_Matrix();
    while (my ($opaque_key, $counts) = each %{ $self->data() }) {
	my (undef, $dst_dotted) = $self->get_key_fields($opaque_key);
	my ($dst_as) = $as_finder->get_as($dst_dotted);
	$dst_as = "NOROUTE" unless defined $dst_as;
	($dst_as) = split(/\s/, $dst_as); # Remove any trailing information
	$as_table->entry_add($dst_as, $counts);
    }
    $as_table->total->add($self->other);
    $as_table->other->add($self->other);
    return $as_table;
}
=cut

sub make_Port_Matrix (@ ) {
    my ($self, $arg_hash_ref) = @_;
    my $protocol = $arg_hash_ref->{'protocol'};
    
    require CAIDA::Tables::Port_Matrix;
    my $port_matrix = new CAIDA::Tables::Port_Matrix();
    while (my ($opaque_key, $counts) = each %{ $self->data() }) {
	my ($s_ip, $d_ip, $ip_proto, $p_ok, $src_port, $dst_port) =
				$self->get_key_fields($opaque_key);
	unless (defined $protocol and $protocol != $ip_proto) {
	    $port_matrix->entry_add($src_port, $dst_port, $counts);
	}
    }
    $port_matrix->total->add($self->other);
    $port_matrix->other->add($self->other);
    return $port_matrix;
}   

# .   .   .   .   .   .   .   .   .   .   .   .   .   .   .   .   .   .   . 
1; ##must be last line in the file
