/* 
 * Copyright 1999, 2000, 2001, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2011, 2012, 2019 The Regents of the University of California
 * All Rights Reserved 
 * 
 * Permission to use, copy, modify and distribute any part of this
 * CoralReef software package for educational, research and non-profit
 * purposes, without fee, and without a written agreement is hereby
 * granted, provided that the above copyright notice, this paragraph
 * and the following paragraphs appear in all copies.
 * 
 * Those desiring to incorporate this into commercial products or use
 * for commercial purposes should contact the Technology Transfer
 * Office, University of California, San Diego, 9500 Gilman Drive, La
 * Jolla, CA 92093-0910, Ph: (858) 534-5815, FAX: (858) 534-7345.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
 * PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
 * DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
 * UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
 * OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
 * OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
 * ANY PATENT, TRADEMARK OR OTHER RIGHTS.
 * 
 * The CoralReef software package is developed by the CoralReef
 * development team at the University of California, San Diego under
 * the Cooperative Association for Internet Data Analysis (CAIDA)
 * Program. Support for this effort is provided by the CAIDA grant
 * NCR-9711092, DARPA NGI Contract N66001-98-2-8922, DARPA NMS Grant
 * N66001-01-1-8909, and by CAIDA members.
 * 
 * Report bugs and suggestions to coral-bugs@caida.org.
 */

/*
 *
 * $Id: coral_atm.h,v 1.17 2019/03/28 23:40:58 kkeys Exp $
 *
 */

/* Using data already stored on iface, prepare the next packet and store it on
 * iface.  Returns 1 if successful, 0 if there's not a complete packet, or -1
 * with errno for an error.  IFACE_TYPE must be #defined by the coral_iface_*.c
 * that includes this file.  This allows coral_IFACE_TYPE_atm_prep_pkt's clock
 * calculations to be inlined.
 */
#define MKID(name)			MKID2(IFACE_TYPE, name)
#define MKID2(iface_type, name)		MKID3(iface_type, name)
#define MKID3(iface_type, name)		coral_ ## iface_type ## _ ## name

static int MKID(atm_prep_pkt)(coral_iface_t *iface)
{
    assert(iface->u.blk.node.data);
    assert(iface->u.blk.node.index < iface->u.blk.node.cell_count);
    if (!coral_aal5_reassemble(iface, coral_iccell(iface), &iface->pkt_result))
	return 0;
    coral_blk_prep_pkt(iface, iface, MKID(clock_timespec));
    return 1;
}

#undef MKID
#undef MKID2
#undef MKID3
#undef IFACE_TYPE

