// Based on Crypto-PAn 1.0 by Jinliang Fan
// Generalized to input up to 128 bits (e.g. IPv6 addresses)

#define _CORAL_PANONYMIZER_CC_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "caida_t.h" /* for uint8_t */
#include "coral_panonymizer.h"

// Constructor
coral_PAnonymizer::coral_PAnonymizer(const uint8_t * key)
{
  //initialize the 128-bit secret key.
  memcpy(m_key, key, 16);
  //initialize the Rijndael cipher. 
  m_rin.init(Rijndael::ECB, Rijndael::Encrypt, key, Rijndael::Key16Bytes);
  //initialize the 128-bit secret pad. The pad is encrypted before being used for padding.
  m_rin.blockEncrypt(key + 16, 128, m_pad);  
}

// Destructor
coral_PAnonymizer::~coral_PAnonymizer() {}

// Anonymization function.  Input should be in network byte order.
void coral_PAnonymizer::anonymize(uint8_t *out, const uint8_t *in,
    size_t size /*size of addr*/, size_t len /*number of bits to anonymize*/)
{
    uint8_t rin_output[16];
    uint8_t rin_input[16];

    for (size_t i = 0; i < size/8; i++) { 
	out[i] = in[i];
    }

    memcpy(rin_input, m_pad, 16);

    // For each prefix with length from 0 to len-1, generate a bit using the
    // Rijndael cipher, which is used as a pseudorandom function here. The
    // bits generated in every round are combined into a pseudorandom
    // one-time-pad.
    for (size_t i = 0; true; i++) { 
	int byte = i / 8;

	//Encryption: The Rijndael cipher is used as pseudorandom function. During each 
	//round, only the first bit of rin_output is used.
	m_rin.blockEncrypt(rin_input, 128, rin_output);	

	//Combination: the bits are combined into a pseudorandom one-time-pad
	//and XOR'd with the original address
	out[byte] ^= ((rin_output[0] & uint8_t(0x80)) >> (i%8));

	if (i == len - 1) break;

	// set up input for next iteration
	uint8_t mask = 0x80 >> ((i)%8);
	// The prefix is taken from orig_addr, the rest is taken from m_pad.
	rin_input[byte] = (rin_input[byte] & ~mask) | (in[byte] & mask);
    }
}
